#include <config.h>

#include <stdio.h>
#include <math.h>
#include <float.h>

#include "ranlib.h"
#include "libhdr.h"

/* Sample from student-t distribution with mean mu,
 * standard deviation sd, and degress of freedom v */
double gen_t(const double mu,const double sd,const double v)
{
	double z,x;
	
	z=snorm();
	x=sqrt(genchi(v));
	return mu+sd*z*sqrt(v)/x;
}

void chfactor(double **a,const int n) 
{
	int i,j,k;
	double sum;
	
	for(k=0;k<n;k++) {
		for(j=k;j<n;j++) {
			sum=a[k][j];
			for(i=k-1;i>=0;i--) sum-=a[k][i]*a[j][i];
			if(j==k) {		
				if(sum<DBL_EPSILON) {
					(void)fprintf(stderr,"chfactor(): Zero pivot found (%d,%f)\n",j,sum);
					exit(EXIT_FAILURE);
				} else a[k][k]=sqrt(sum);
			} else a[j][k]=sum/a[k][k];
		}
 	}
}

void chsolve(double **a,const int n,double *x) /*@modifies *x; @*/
{
	int i,j;
	double tmp;
	
	for(i=0;i<n;i++) {
		tmp=x[i];
		for(j=i-1;j>=0;j--) tmp-=a[i][j]*x[j];
		x[i]=tmp/a[i][i];
	}
	for(i=n-1;i>=0;i--) {
		tmp=x[i];
		for(j=i+1;j<n;j++) tmp-=a[j][i]*x[j];
		x[i]=tmp/a[i][i];
	}
}

