/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                   Simon Heath - MSKCC                                    *
 *                                                                          *
 *                       August 2000                                        *
 *                                                                          *
 * calc_var_locus.c:                                                        *
 *                                                                          *
 * Calculate variance contributed by a trait locus                          *
 *                                                                          *
 ****************************************************************************/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>

#include "loki.h"
#include "utils.h"
#include "calc_var_locus.h"

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "calc_var_locus"
double calc_var_locus(int locus)
{
	int i,k,df;
	double m,nn,*eff,s,z;
	int *gt;
	static int *ct,ct_size;
	
	if(locus==n_markers) {
		if(ct) free(ct);
		return 0.0;
	}
	if(locus<0) {
		eff=tlocus[-1-locus].eff;
		gt=tlocus[-1-locus].gt;
		k=tlocus[-1-locus].n_alleles;
	} else {
		eff=marker[locus].mterm->eff;
		gt=marker[locus].gt;
		k=marker[locus].n_alleles;
	}
	df=k*(k+1)/2;
	if(df<2) return 0.0;
	if(!ct) {
		ct_size=df;
		if(!(ct=malloc(sizeof(int)*ct_size))) ABT_FUNC(MMsg);
	} else if(df>ct_size) {
		ct_size=df;
		if(!(ct=realloc(ct,sizeof(int)*ct_size))) ABT_FUNC(MMsg);
	}
	for(i=0;i<df;i++) ct[i]=0;
	nn=0.0;
	for(i=0;i<ped_size;i++) if(id_array[i].res) {
		k=gt[i]-1;
		ct[k]++;
		nn+=1.0;
	}
	m=0.0;
	for(i=1;i<df;i++) m+=eff[i-1]*(double)ct[i];
	m=nn?m/nn:0.0;
	s=m*m*(double)ct[0];
	for(i=1;i<df;i++) {
		z=eff[i-1]-m;
		s+=z*z*(double)ct[i];
	}
	return nn?s/nn:0.0;
}

