%{
/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                       July 1997                                          *
 *                                                                          *
 * param_lex.l:                                                             *
 *                                                                          *
 * flex (NOT lex!) source for parameter file lexer.                         *
 *                                                                          *
 ****************************************************************************/

#include <config.h>
#include <stdlib.h>
#include <string.h>

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

#include "param_parse.h"
#include "utils.h"

extern void yyerror(char *s),print_scan_err(char *fmt, ...);
extern int scan_error_n;
	
static int token,i;
int tokenpos,lineno=1,lineno1=1;
char linebuf[512],linebuf1[512];
static int symbol_lookup(const char *p);
static char *get_string(const char *p);
	
%}

%x COMMENT
%s ISTATE

%%

<INITIAL>.*\n {
    strncpy(linebuf,yytext,512);
    strcpy(linebuf1,linebuf);
    yyless((int)0);
    BEGIN ISTATE;
    }

[\t ] {tokenpos+=yyleng;}

\n.* { 
    lineno++;
    tokenpos=0;
    strncpy(linebuf,yytext+1,512);
    yyless((int)1);
    }	

"/*" BEGIN(COMMENT);
<COMMENT>. {tokenpos+=yyleng;}
<COMMENT>\n {lineno++; tokenpos=0;}
<COMMENT>"*/" {BEGIN(ISTATE); tokenpos+=yyleng;}

#.*$ ;

\"[^\"\n]*[\n\"] { tokenpos+=yyleng;
	          yylval.string=get_string(yytext+1);
	          i=strlen(yylval.string);
				 if(i) yylval.string[i-1]='\0';
	          return STRING; }

\'[^\'\n]*[\n\'] { tokenpos+=yyleng;
	          yylval.string=get_string(yytext+1);
	          i=strlen(yylval.string);
				 if(i) yylval.string[i-1]='\0';
	          return STRING; }

[a-zA-Z][a-zA-Z0-9_]* { token=symbol_lookup(yytext);
				    tokenpos+=yyleng;
	                   if(token==STRING) yylval.string=get_string(yytext);
				    return token; }

[+-]?([0-9]*\.[0-9]+)|(([0-9]+|([0-9]*\.[0-9]+))E[-+]?[0-9]+) {
         tokenpos+=yyleng; yylval.rvalue=atof(yytext); return REAL; }

[+-]?[0-9]+ { tokenpos+=yyleng; yylval.value=atoi(yytext); return INTEGER; }

. { tokenpos+=yyleng; return yytext[0]; }

%%

void yy_cleanup(void)
{
	yy_delete_buffer(yy_current_buffer);
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "get_string"
static char *get_string(const char *s)
{
	char *s1;
	size_t size;
	
	size=strlen(s)+1;
	if(!(s1=malloc(size))) ABT_FUNC(MMsg);
	memcpy(s1,s,size);
	return s1;
}

static int symbol_lookup(const char *p)
{
	static char *Coms[] = {"RESIDUAL","GENETIC","VARIANCE","POSITION","FREQUENCY",
		"START","ITERATIONS","SAMPLE","FROM","OUTPUT","MEAN","MAP","TOTAL",
		"SEED","FILE","SEEDFILE","TRAIT","LOCI","SET","ESTIMATE","IBD",
		"GROUP","ORDER","MALE","FEMALE","LIMIT","PHENOTYPE",
		"PHENOTYPES","PHENO","GENOTYPE","GENOTYPES","GENO","COUNTS","DUMP",
		"TYPE","ANALYZE","NORMAL","STUDENT_T","HAPLO","HAPLOTYPES",(char *)0};
	
	static int Com_token[] = {RESIDUAL,GENETIC,VARIANCE,POSITION,FREQUENCY,
		START,ITERATIONS,SAMPLE,FROM,OUTPUT,MEAN,MAP,TOTAL,
		SEED,SFILE,SEEDFILE,TRAIT,LOCI,SET,ESTIMATE,IBD,
		GROUP,ORDER,MALE,FEMALE,LIMIT,PHENO,PHENO,PHENO,GENO,GENO,GENO,
		COUNTS,DUMP,TYPE,ANALYZE,NORMAL,STUDENT_T,HAPLO,HAPLO,SYSTEM_VAR,STRING};

	static char *Syst[] = {"NO_OVERDOMINANT","TAU_BETA","TAU_MODE","CENSOR_MODE",
		"DEBUG_LEVEL","LM_RATIO","L_PROB","L_CYCLES","PEEL_TRACE","BACKUPS",
		"SI_MODE",(char *)0};

	int i=0,j;
	
	while(Coms[i])	{
		if(!strcasecmp(Coms[i],p)) break;
		i++;
	}
	if(Com_token[i]==SYSTEM_VAR) {
		i++;
		j=0;
		while(Syst[j]) {
			if(!strcasecmp(Syst[j],p))	{
				yylval.value=j;
				i--;
				break;
			}
			j++;
		}
	}
	return Com_token[i];
}
