#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define YYPREFIX "yy"
#line 2 "param_parse.y"
/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *             Simon Heath - University of Washington                       *
 *                                                                          *
 *                       July 1997                                          *
 *                                                                          *
 * param_parse.y:                                                           *
 *                                                                          *
 * yacc source for parameter file parser.                                   *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "utils.h"
#include "loki_scan.h"
#include "shared_peel.h"
#include "loki.h"
	
extern FILE *yyin;
double residual_var,residual_var_limit,additive_var,additive_var_limit,grand_mean,tloci_mean;
int res_var_set,add_var_set,grand_mean_set,num_iter,max_tloci=16,min_tloci,sex_map;
int tloci_mean_set,debug_level,start_tloci=-1,dump_freq,output_type=DEFAULT_OUTPUT_TYPE;
int sample_from[2]={0,0};
int sample_freq[2]={1,1};
static int start_flag=1;
static struct Marker *freq_marker;
static int check_variance( double);
static void set_position(struct lk_variable *, double, double);
static struct lk_variable *find_var( char *, int, int);
static struct Marker *check_marker(struct lk_variable *);
static int find_allele( char *, struct Marker *,int, int);
static int find_group( char *,int, int);
static void set_output_gen(char *,char *);
static struct IBD_List *add_ibd_list(double,struct IBD_List *);
static void set_freq(struct Marker *, double,int);
static void set_map_range( char *,double,double, int);
static void set_tloci(int,int);	
static void set_ibd_list( char *,struct IBD_List *);
static void set_output(struct lk_variable *);
static void set_group_order( int);
static void set_analyze(char *);
static struct Variable *group_var;
static int group_ptr,*group_order,group_counter,freq_allele,c_flag;
extern void yyerror(char *s),print_scan_err(char *fmt, ...);
extern int yyparse(void),yylex(void),lineno,lineno1,tokenpos;
extern char *yytext,linebuf[];
char *Output_Phen,*Outputfile,*Dumpfile,*Freqfile,*Haplofile;
struct output_gen *Output_Gen;

static int max_scan_errors=30;
static int scan_warn_n,max_scan_warnings=30;
int scan_error_n,output_haplo;

struct id_data syst_var[NUM_SYSTEM_VAR];

#line 68 "param_parse.y"
typedef union  {
	char *string;
	int value;
	double rvalue;
	struct IBD_List *rlist;
	struct lk_variable *lk_var;
} YYSTYPE;
#line 90 "y.tab.c"
#define YYERRCODE 256
#define RESIDUAL 257
#define GENETIC 258
#define VARIANCE 259
#define POSITION 260
#define FREQUENCY 261
#define START 262
#define MEAN 263
#define ITERATIONS 264
#define SAMPLE 265
#define FROM 266
#define OUTPUT 267
#define MAP 268
#define TOTAL 269
#define SEED 270
#define SFILE 271
#define SEEDFILE 272
#define TRAIT 273
#define LOCI 274
#define SET 275
#define SYSTEM_VAR 276
#define ESTIMATE 277
#define IBD 278
#define GROUP 279
#define ORDER 280
#define MALE 281
#define FEMALE 282
#define LIMIT 283
#define PHENO 284
#define GENO 285
#define COUNTS 286
#define DUMP 287
#define TYPE 288
#define ANALYZE 289
#define NORMAL 290
#define STUDENT_T 291
#define HAPLO 292
#define STRING 293
#define INTEGER 294
#define REAL 295
const short yylhs[] = {                                        -1,
    7,    0,    0,    8,    0,    0,    6,    6,   11,    6,
    9,    9,    9,    9,    9,   10,   10,   10,   10,   10,
   10,   10,   10,   10,   10,   10,   10,   18,   18,   18,
   18,   23,   23,   23,   24,   24,   29,   29,   30,   30,
   31,   28,   21,   21,   21,   21,   21,   20,   20,   20,
   20,   20,   20,   20,   20,   20,   20,   20,   22,   22,
   22,   22,   17,   19,   19,   19,   19,   19,   19,   19,
   19,   19,   19,   19,   19,   19,   19,   19,   12,   12,
   12,   26,   26,   13,   27,   27,   16,   14,   14,   34,
   15,   36,   15,   37,   15,    3,    3,   32,   32,   33,
   39,   25,    5,    5,    5,   38,   38,    4,    4,    4,
   41,   35,   42,   35,   40,   40,   40,   40,    2,    2,
    1,    1,
};
const short yylen[] = {                                         2,
    0,    2,    1,    0,    3,    2,    1,    1,    0,    3,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    3,    5,    3,
    5,    3,    3,    3,    5,    3,    1,    1,    1,    3,
    0,    3,    2,    4,    3,    5,    2,    5,    6,    6,
    3,    5,    4,    4,    3,    5,    4,    4,    3,    4,
    5,    4,    2,    3,    5,    3,    3,    5,    3,    3,
    5,    2,    3,    3,    3,    3,    3,    2,    3,    3,
    3,    4,    4,    3,    4,    4,    2,    3,    5,    0,
    4,    0,    5,    0,    4,    1,    4,    1,    3,    1,
    0,    4,    1,    1,    1,    1,    3,    1,    1,    1,
    0,    4,    0,    5,    1,    1,    3,    3,    1,    3,
    1,    1,
};
const short yydefred[] = {                                      0,
    3,    0,    0,    6,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   94,    0,   41,    2,    7,
    8,   11,   12,   13,   14,   15,   16,   17,   18,   19,
   20,   21,   22,   23,   24,   25,   26,   27,    5,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  122,  121,
   87,   63,    0,    0,    0,    0,    0,    0,    0,    0,
   98,    0,    0,    0,    0,    0,    0,    0,   47,    0,
    0,    0,    0,    0,  101,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   80,   81,   79,    0,   84,    0,
    0,  100,    0,    0,    0,    0,    0,    0,   10,    0,
    0,   66,    0,   74,   69,    0,   73,   77,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   32,   33,
   34,    0,  119,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   76,   75,   38,   37,   39,    0,   82,   85,
    0,    0,  110,  108,  109,    0,    0,    0,    0,   60,
    0,    0,    0,    0,    0,    0,   99,    0,    0,    0,
   53,   54,    0,   44,   62,    0,    0,    0,  105,  103,
  104,  106,    0,   57,    0,   58,    0,   83,   86,    0,
    0,   97,   89,  111,    0,    0,   31,   68,   29,   65,
   71,   56,   48,   52,   46,   61,    0,  120,    0,    0,
    0,   40,    0,  113,  107,   49,   50,  116,  115,    0,
    0,    0,    0,  118,  117,
};
const short yydgoto[] = {                                       2,
   97,  134,  102,  156,  182,   29,    3,    5,   30,   31,
   58,   32,   33,   34,   35,   36,   37,   38,   39,   40,
   41,   42,   43,   44,   45,   46,   47,   48,  147,  148,
   93,   72,  103,   54,  157,   55,   90,  183,  135,  220,
  213,  221,
};
const short yysindex[] = {                                   -239,
    0, -205, -249,    0, -249, -172, -160, -183,    0, -200,
 -204, -187, -211, -232, -262, -184, -228, -179, -134, -271,
 -158, -139, -126, -125, -162,    0, -217,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -189,
 -191,  104, -204, -183, -141, -148, -124, -186,    0,    0,
    0,    0, -146, -145, -177, -142, -140, -137, -136, -133,
    0,  108, -204, -204, -204, -114, -109, -132,    0,  118,
 -236, -165, -204, -170,    0, -244, -234,  -93,  -92, -183,
 -123, -119, -213, -204,    0,    0,    0, -204,    0, -118,
  124,    0, -167, -183,  125, -117,  -89,  -84,    0,  134,
  135,    0,  136,    0,    0,  137,    0,    0, -183,  138,
  139,  140, -204, -204,  141, -108, -204,  143,    0,    0,
    0,  144,    0,  146, -156, -204, -204, -204, -204, -204,
 -204, -167,    0,    0,    0,    0,    0,  147,    0,    0,
  151, -204,    0,    0,    0,  149, -167, -167, -100,    0,
 -182, -204,  -99,  -98,  -97,  -94,    0, -204, -204, -204,
    0,    0,  -96,    0,    0,  -91, -204, -204,    0,    0,
    0,    0,  156,    0,  157,    0,  158,    0,    0, -167,
 -213,    0,    0,    0,  160, -167,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  146,    0, -156, -204,
 -204,    0,  -42,    0,    0,    0,    0,    0,    0,  163,
  -42,  -40,  163,    0,    0,
};
const short yyrindex[] = {                                    828,
    0,  861,    0,    0,    0,    0,    0,    0, -229, -175,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    1,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  121,
    0,  155,    0,    0,    0,    0,    0,    0,    0,  189,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  223,    0,    0,    0,  260,    0,    0,    0,    0,  300,
  340,    0,  380,    0,    0,  414,    0,    0,    0,  448,
    0,  482,    0,    0,  519,    0,    0,  553,    0,    0,
    0,    0,    0,  588,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  624,    0,    0,
    0,    0,    0,    0,    0,    0,  658,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  692,    0,    0,    0,    0,    0,    0,  726,
    0,    0,    0,    0,    0,  760,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  795,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   41,
    0,    0,   81,    0,    0,
};
const short yygindex[] = {                                      0,
   -5,   32,   -4, -154,    2,  205,    0,    0,  154,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   22,    0,
    0,    0,  -28,    0,  -95,    0,    0,    0,    0,   -7,
    0,    0,
};
#define YYTABLESIZE 1150
const short yytable[] = {                                     218,
   96,  224,  195,   53,   82,   61,   73,    6,    7,   71,
    8,    9,   10,   11,   12,   13,    1,   14,   15,   16,
   17,   83,   18,   19,  136,   20,  127,   21,   65,   22,
   74,   23,   24,   25,  138,  195,   26,   27,   66,   28,
  112,  195,   78,   91,   96,   99,  190,  101,  137,   63,
    4,   67,   68,   92,   64,   69,   92,  128,  139,   70,
   52,  142,  196,   90,  145,   79,   56,  120,  121,  122,
  107,  108,   57,    8,    9,  158,   11,  131,  133,  146,
  114,    9,    9,   75,    9,    9,   50,    9,  149,   59,
   60,   98,  150,   94,   88,   89,   76,   77,   51,   26,
   95,   96,   59,   60,   59,   60,   62,   95,   96,   52,
    9,   59,   60,   80,  167,  112,  113,  171,  172,   84,
   78,  175,  132,   59,   60,  153,  154,  155,  129,  130,
  184,  185,  186,  187,  188,  189,  179,  180,  181,   81,
   85,   86,   87,  100,  104,  105,  193,  110,  111,  106,
  114,  119,  115,  123,   72,  116,   99,  117,  124,  118,
  125,  126,  202,  203,  204,  140,  141,  152,  159,  161,
  143,  133,  208,  144,  162,  151,  160,  163,  164,  165,
  166,  168,  169,  170,  173,  174,  176,  177,   43,  178,
  191,  192,  194,  197,  198,  199,  200,  205,  201,  209,
  210,  211,  206,  214,  216,  217,  222,  219,  207,   49,
  215,  109,  212,  223,    0,  219,  225,    0,    0,    0,
    0,    0,   88,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   59,   60,   59,   60,    0,   96,   96,   96,   30,
   96,   96,   96,   96,   96,   96,    0,   96,   96,   96,
   96,    0,   96,   96,    0,   96,    0,   96,    0,   96,
    0,   96,   96,   96,    0,    0,   96,   96,    0,   96,
    0,    0,    0,   96,   96,   96,  112,  112,  112,   67,
  112,  112,  112,  112,  112,  112,    0,  112,  112,  112,
  112,    0,  112,  112,    0,  112,    0,  112,    0,  112,
    0,  112,  112,  112,    0,    0,  112,  112,    0,  112,
    0,    0,    0,  112,  112,  112,  114,  114,  114,   28,
  114,  114,  114,  114,  114,  114,    0,  114,  114,  114,
  114,    0,  114,  114,    0,  114,    0,  114,    0,  114,
    0,  114,  114,  114,    0,    0,  114,  114,    0,  114,
    0,    0,    0,  114,  114,  114,   78,   78,   78,   64,
   78,   78,   78,   78,   78,   78,    0,   78,   78,   78,
   78,    0,   78,   78,    0,   78,    0,   78,    0,   78,
    0,   78,   78,   78,    0,    0,   78,   78,    0,   78,
   72,   72,   72,   70,   72,   72,   72,   72,   72,   72,
    0,   72,   72,   72,   72,    0,   72,   72,    0,   72,
    0,   72,    0,   72,    0,   72,   72,   72,    0,    0,
   72,   72,    0,   72,   43,   43,   43,   55,   43,   43,
   43,   43,   43,   43,    0,   43,   43,   43,   43,    0,
   43,   43,    0,   43,    0,   43,    0,   43,    0,   43,
   43,   43,    0,    0,   43,   43,    0,   43,   88,   88,
   88,   51,   88,   88,   88,   88,   88,   88,    0,   88,
   88,   88,   88,    0,   88,   88,    0,   88,    0,   88,
    0,   88,    0,   88,   88,   88,    0,    0,   88,   88,
    0,   88,    0,    0,    0,   30,   30,   30,   45,   30,
   30,   30,   30,   30,   30,    0,   30,   30,   30,   30,
    0,   30,   30,    0,   30,    0,   30,    0,   30,    0,
   30,   30,   30,    0,    0,   30,   30,    0,   30,    0,
    0,    0,   59,    0,    0,   67,   67,   67,    0,   67,
   67,   67,   67,   67,   67,    0,   67,   67,   67,   67,
    0,   67,   67,    0,   67,    0,   67,    0,   67,    0,
   67,   67,   67,    0,    0,   67,   67,   36,   67,    0,
    0,    0,    0,    0,    0,   28,   28,   28,    0,   28,
   28,   28,   28,   28,   28,    0,   28,   28,   28,   28,
    0,   28,   28,    0,   28,    0,   28,    0,   28,    0,
   28,   28,   28,   42,    0,   28,   28,    0,   28,    0,
    0,    0,    0,    0,    0,   64,   64,   64,    0,   64,
   64,   64,   64,   64,   64,    0,   64,   64,   64,   64,
    0,   64,   64,    0,   64,    0,   64,   91,   64,    0,
   64,   64,   64,    0,    0,   64,   64,    0,   64,   70,
   70,   70,    0,   70,   70,   70,   70,   70,   70,    0,
   70,   70,   70,   70,    0,   70,   70,    0,   70,    0,
   70,  102,   70,    0,   70,   70,   70,    0,    0,   70,
   70,    0,   70,   55,   55,   55,    0,   55,   55,   55,
   55,   55,   55,    0,   55,   55,   55,   55,    0,   55,
   55,    0,   55,    0,   55,   95,   55,    0,   55,   55,
   55,    0,    0,   55,   55,    0,   55,   51,   51,   51,
    0,   51,   51,   51,   51,   51,   51,    0,   51,   51,
   51,   51,    0,   51,   51,    0,   51,    0,   51,   93,
   51,    0,   51,   51,   51,    0,    0,   51,   51,    0,
   51,    0,    0,    0,   45,   45,   45,    0,   45,   45,
   45,   45,   45,   45,    0,   45,   45,   45,   45,    0,
   45,   45,    0,   45,   35,   45,    0,   45,    0,   45,
   45,   45,    0,    0,   45,   45,    0,   45,   59,   59,
   59,    0,   59,   59,   59,   59,   59,   59,    0,   59,
   59,   59,   59,    0,   59,   59,    0,   59,    0,   59,
    0,   59,    0,   59,   59,   59,    0,    0,   59,   59,
    0,   59,    0,   36,   36,   36,    0,   36,   36,   36,
   36,   36,   36,    0,   36,   36,   36,   36,    0,   36,
   36,    0,   36,    0,   36,    0,   36,    0,   36,   36,
   36,    0,    0,   36,   36,    0,   36,    0,    0,   42,
   42,   42,    0,   42,   42,   42,   42,   42,   42,    0,
   42,   42,   42,   42,    0,   42,   42,    0,   42,    0,
   42,    0,   42,    0,   42,   42,   42,    0,    0,   42,
   42,    0,   42,   91,   91,   91,    0,   91,   91,   91,
   91,   91,   91,    0,   91,   91,   91,   91,    0,   91,
   91,    0,   91,    0,   91,    0,   91,    0,   91,   91,
   91,    0,    0,   91,   91,    0,   91,  102,  102,  102,
    0,  102,  102,  102,  102,  102,  102,    0,  102,  102,
  102,  102,    0,  102,  102,    0,  102,    0,  102,    0,
  102,    0,  102,  102,  102,    0,    0,  102,  102,    0,
  102,   95,   95,   95,    0,   95,   95,   95,   95,   95,
   95,    0,   95,   95,   95,   95,    0,   95,   95,    0,
   95,    0,   95,    0,   95,    0,   95,   95,   95,    0,
    0,   95,   95,    0,   95,   93,   93,   93,    0,   93,
   93,   93,   93,   93,   93,    0,   93,   93,   93,   93,
    0,   93,   93,    0,   93,    0,   93,    0,   93,    0,
   93,   93,   93,    0,    0,   93,   93,    0,   93,    0,
   35,   35,   35,    0,   35,   35,   35,   35,   35,   35,
    0,   35,   35,   35,   35,    0,   35,   35,    0,   35,
    0,   35,    0,   35,    0,   35,   35,   35,    0,    0,
   35,   35,    0,   35,    1,    1,    0,    1,    1,    1,
    1,    1,    1,    0,    1,    1,    1,    1,    0,    1,
    1,    0,    1,    0,    1,    0,    1,    0,    1,    1,
    1,    0,    0,    1,    1,    0,    1,    4,    4,    0,
    4,    4,    4,    4,    4,    4,    0,    4,    4,    4,
    4,    0,    4,    4,    0,    4,    0,    4,    0,    4,
    0,    4,    4,    4,    0,    0,    4,    4,    0,    4,
};
const short yycheck[] = {                                      42,
    0,   42,  157,    8,  276,   11,  269,  257,  258,   14,
  260,  261,  262,  263,  264,  265,  256,  267,  268,  269,
  270,  293,  272,  273,  269,  275,  263,  277,  261,  279,
  293,  281,  282,  283,  269,  190,  286,  287,  271,  289,
    0,  196,  271,  261,   44,   51,  142,   53,  293,  261,
  256,  284,  285,  271,  266,  288,  286,  294,  293,  292,
  293,   90,  158,  293,  278,  294,  267,   73,   74,   75,
  257,  258,  273,  260,  261,  104,  263,   83,   84,  293,
    0,  257,  258,  268,  260,  261,  259,  263,   94,  294,
  295,  283,   98,  283,  257,  258,  281,  282,  259,  286,
  290,  291,  294,  295,  294,  295,  294,  290,  291,  293,
  286,  294,  295,  293,  119,  293,  294,  123,  124,  278,
    0,  127,  293,  294,  295,  293,  294,  295,  294,  295,
  136,  137,  138,  139,  140,  141,  293,  294,  295,  274,
  280,  268,  268,   40,  286,  294,  152,  294,  294,  274,
  293,   44,  293,  268,    0,  293,  162,  294,  268,  293,
  293,   44,  168,  169,  170,  259,  259,   44,   44,  259,
  294,  177,  178,  293,  259,  294,  294,   44,   44,   44,
   44,   44,   44,   44,   44,  294,   44,   44,    0,   44,
   44,   41,   44,  294,  294,  294,  294,  294,  293,   44,
   44,   44,  294,   44,  210,  211,   44,  213,  177,    5,
  209,   58,  191,  221,   -1,  221,  222,   -1,   -1,   -1,
   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  294,  295,  294,  295,   -1,  256,  257,  258,    0,
  260,  261,  262,  263,  264,  265,   -1,  267,  268,  269,
  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,
   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,
   -1,   -1,   -1,  293,  294,  295,  256,  257,  258,    0,
  260,  261,  262,  263,  264,  265,   -1,  267,  268,  269,
  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,
   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,
   -1,   -1,   -1,  293,  294,  295,  256,  257,  258,    0,
  260,  261,  262,  263,  264,  265,   -1,  267,  268,  269,
  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,
   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,
   -1,   -1,   -1,  293,  294,  295,  256,  257,  258,    0,
  260,  261,  262,  263,  264,  265,   -1,  267,  268,  269,
  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,
   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,
  256,  257,  258,    0,  260,  261,  262,  263,  264,  265,
   -1,  267,  268,  269,  270,   -1,  272,  273,   -1,  275,
   -1,  277,   -1,  279,   -1,  281,  282,  283,   -1,   -1,
  286,  287,   -1,  289,  256,  257,  258,    0,  260,  261,
  262,  263,  264,  265,   -1,  267,  268,  269,  270,   -1,
  272,  273,   -1,  275,   -1,  277,   -1,  279,   -1,  281,
  282,  283,   -1,   -1,  286,  287,   -1,  289,  256,  257,
  258,    0,  260,  261,  262,  263,  264,  265,   -1,  267,
  268,  269,  270,   -1,  272,  273,   -1,  275,   -1,  277,
   -1,  279,   -1,  281,  282,  283,   -1,   -1,  286,  287,
   -1,  289,   -1,   -1,   -1,  256,  257,  258,    0,  260,
  261,  262,  263,  264,  265,   -1,  267,  268,  269,  270,
   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,   -1,
  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,   -1,
   -1,   -1,    0,   -1,   -1,  256,  257,  258,   -1,  260,
  261,  262,  263,  264,  265,   -1,  267,  268,  269,  270,
   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,   -1,
  281,  282,  283,   -1,   -1,  286,  287,    0,  289,   -1,
   -1,   -1,   -1,   -1,   -1,  256,  257,  258,   -1,  260,
  261,  262,  263,  264,  265,   -1,  267,  268,  269,  270,
   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,   -1,
  281,  282,  283,    0,   -1,  286,  287,   -1,  289,   -1,
   -1,   -1,   -1,   -1,   -1,  256,  257,  258,   -1,  260,
  261,  262,  263,  264,  265,   -1,  267,  268,  269,  270,
   -1,  272,  273,   -1,  275,   -1,  277,    0,  279,   -1,
  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,  256,
  257,  258,   -1,  260,  261,  262,  263,  264,  265,   -1,
  267,  268,  269,  270,   -1,  272,  273,   -1,  275,   -1,
  277,    0,  279,   -1,  281,  282,  283,   -1,   -1,  286,
  287,   -1,  289,  256,  257,  258,   -1,  260,  261,  262,
  263,  264,  265,   -1,  267,  268,  269,  270,   -1,  272,
  273,   -1,  275,   -1,  277,    0,  279,   -1,  281,  282,
  283,   -1,   -1,  286,  287,   -1,  289,  256,  257,  258,
   -1,  260,  261,  262,  263,  264,  265,   -1,  267,  268,
  269,  270,   -1,  272,  273,   -1,  275,   -1,  277,    0,
  279,   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,
  289,   -1,   -1,   -1,  256,  257,  258,   -1,  260,  261,
  262,  263,  264,  265,   -1,  267,  268,  269,  270,   -1,
  272,  273,   -1,  275,    0,  277,   -1,  279,   -1,  281,
  282,  283,   -1,   -1,  286,  287,   -1,  289,  256,  257,
  258,   -1,  260,  261,  262,  263,  264,  265,   -1,  267,
  268,  269,  270,   -1,  272,  273,   -1,  275,   -1,  277,
   -1,  279,   -1,  281,  282,  283,   -1,   -1,  286,  287,
   -1,  289,   -1,  256,  257,  258,   -1,  260,  261,  262,
  263,  264,  265,   -1,  267,  268,  269,  270,   -1,  272,
  273,   -1,  275,   -1,  277,   -1,  279,   -1,  281,  282,
  283,   -1,   -1,  286,  287,   -1,  289,   -1,   -1,  256,
  257,  258,   -1,  260,  261,  262,  263,  264,  265,   -1,
  267,  268,  269,  270,   -1,  272,  273,   -1,  275,   -1,
  277,   -1,  279,   -1,  281,  282,  283,   -1,   -1,  286,
  287,   -1,  289,  256,  257,  258,   -1,  260,  261,  262,
  263,  264,  265,   -1,  267,  268,  269,  270,   -1,  272,
  273,   -1,  275,   -1,  277,   -1,  279,   -1,  281,  282,
  283,   -1,   -1,  286,  287,   -1,  289,  256,  257,  258,
   -1,  260,  261,  262,  263,  264,  265,   -1,  267,  268,
  269,  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,
  279,   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,
  289,  256,  257,  258,   -1,  260,  261,  262,  263,  264,
  265,   -1,  267,  268,  269,  270,   -1,  272,  273,   -1,
  275,   -1,  277,   -1,  279,   -1,  281,  282,  283,   -1,
   -1,  286,  287,   -1,  289,  256,  257,  258,   -1,  260,
  261,  262,  263,  264,  265,   -1,  267,  268,  269,  270,
   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,   -1,
  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,   -1,
  256,  257,  258,   -1,  260,  261,  262,  263,  264,  265,
   -1,  267,  268,  269,  270,   -1,  272,  273,   -1,  275,
   -1,  277,   -1,  279,   -1,  281,  282,  283,   -1,   -1,
  286,  287,   -1,  289,  257,  258,   -1,  260,  261,  262,
  263,  264,  265,   -1,  267,  268,  269,  270,   -1,  272,
  273,   -1,  275,   -1,  277,   -1,  279,   -1,  281,  282,
  283,   -1,   -1,  286,  287,   -1,  289,  257,  258,   -1,
  260,  261,  262,  263,  264,  265,   -1,  267,  268,  269,
  270,   -1,  272,  273,   -1,  275,   -1,  277,   -1,  279,
   -1,  281,  282,  283,   -1,   -1,  286,  287,   -1,  289,
};
#define YYFINAL 2
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 295
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,"'('","')'","'*'",0,"','",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"RESIDUAL",
"GENETIC","VARIANCE","POSITION","FREQUENCY","START","MEAN","ITERATIONS",
"SAMPLE","FROM","OUTPUT","MAP","TOTAL","SEED","SFILE","SEEDFILE","TRAIT","LOCI",
"SET","SYSTEM_VAR","ESTIMATE","IBD","GROUP","ORDER","MALE","FEMALE","LIMIT",
"PHENO","GENO","COUNTS","DUMP","TYPE","ANALYZE","NORMAL","STUDENT_T","HAPLO",
"STRING","INTEGER","REAL",
};
const char * const yyrule[] = {
"$accept : parmfile",
"$$1 :",
"parmfile : $$1 command1",
"parmfile : error",
"$$2 :",
"parmfile : parmfile $$2 command1",
"parmfile : parmfile error",
"command1 : command",
"command1 : command_a",
"$$3 :",
"command1 : START $$3 command",
"command : resvarcommand",
"command : addvarcommand",
"command : positioncommand",
"command : frequencycommand",
"command : meancommand",
"command_a : itercommand",
"command_a : samplecommand",
"command_a : outputcommand",
"command_a : mapcommand",
"command_a : seedcommand",
"command_a : tlocicommand",
"command_a : setcommand",
"command_a : ibdcommand",
"command_a : groupcommand",
"command_a : limitresvarcommand",
"command_a : limitaddvarcommand",
"command_a : analyzecommand",
"samplecommand : SAMPLE FROM INTEGER",
"samplecommand : SAMPLE FROM INTEGER ',' INTEGER",
"samplecommand : START OUTPUT INTEGER",
"samplecommand : START OUTPUT INTEGER ',' INTEGER",
"setcommand : SET SYSTEM_VAR INTEGER",
"setcommand : SET SYSTEM_VAR REAL",
"setcommand : SET STRING rnum",
"ibdcommand : ESTIMATE IBD STRING ',' ibdlist",
"ibdcommand : ESTIMATE IBD ibdlist",
"analyzecom : STRING",
"analyzecom : IBD",
"analyzelist : analyzecom",
"analyzelist : analyzelist ',' analyzecom",
"$$4 :",
"analyzecommand : ANALYZE $$4 analyzelist",
"seedcommand : SEEDFILE STRING",
"seedcommand : SEEDFILE STRING ',' INTEGER",
"seedcommand : SEED SFILE STRING",
"seedcommand : SEED SFILE STRING ',' INTEGER",
"seedcommand : SEED INTEGER",
"mapcommand : MAP STRING rnum ',' rnum",
"mapcommand : MALE MAP STRING rnum ',' rnum",
"mapcommand : FEMALE MAP STRING rnum ',' rnum",
"mapcommand : TOTAL MAP rnum",
"mapcommand : TOTAL MAP rnum ',' rnum",
"mapcommand : TOTAL MALE MAP rnum",
"mapcommand : TOTAL FEMALE MAP rnum",
"mapcommand : MAP TOTAL rnum",
"mapcommand : MAP TOTAL rnum ',' rnum",
"mapcommand : MALE MAP TOTAL rnum",
"mapcommand : FEMALE MAP TOTAL rnum",
"tlocicommand : TRAIT LOCI INTEGER",
"tlocicommand : START TRAIT LOCI INTEGER",
"tlocicommand : TRAIT LOCI INTEGER ',' INTEGER",
"tlocicommand : TRAIT LOCI MEAN rnum",
"itercommand : ITERATIONS INTEGER",
"outputcommand : OUTPUT FREQUENCY INTEGER",
"outputcommand : OUTPUT FREQUENCY INTEGER ',' INTEGER",
"outputcommand : OUTPUT FREQUENCY STRING",
"outputcommand : SAMPLE FREQUENCY INTEGER",
"outputcommand : SAMPLE FREQUENCY INTEGER ',' INTEGER",
"outputcommand : OUTPUT PHENO STRING",
"outputcommand : OUTPUT GENO STRING",
"outputcommand : OUTPUT GENO STRING ',' STRING",
"outputcommand : OUTPUT lkvarlist",
"outputcommand : OUTPUT TYPE INTEGER",
"outputcommand : OUTPUT SFILE STRING",
"outputcommand : DUMP SFILE STRING",
"outputcommand : DUMP FREQUENCY INTEGER",
"outputcommand : OUTPUT HAPLO STRING",
"outputcommand : OUTPUT HAPLO",
"resvarcommand : RESIDUAL VARIANCE rnum",
"resvarcommand : RESIDUAL VARIANCE NORMAL",
"resvarcommand : RESIDUAL VARIANCE STUDENT_T",
"limitresvarcommand : RESIDUAL VARIANCE LIMIT rnum",
"limitresvarcommand : LIMIT RESIDUAL VARIANCE rnum",
"addvarcommand : GENETIC VARIANCE rnum",
"limitaddvarcommand : GENETIC VARIANCE LIMIT rnum",
"limitaddvarcommand : LIMIT GENETIC VARIANCE rnum",
"meancommand : MEAN rnum",
"positioncommand : POSITION lkvar rnum",
"positioncommand : POSITION lkvar rnum ',' rnum",
"$$5 :",
"frequencycommand : FREQUENCY $$5 lkmarker freqlist",
"$$6 :",
"frequencycommand : FREQUENCY $$6 COUNTS lkmarker freqlist",
"$$7 :",
"frequencycommand : COUNTS $$7 lkmarker freqlist",
"lkvar : STRING",
"lkvar : STRING '(' INTEGER ')'",
"lkvarlist : lkvar",
"lkvarlist : lkvarlist ',' lkvar",
"lkmarker : lkvar",
"$$8 :",
"groupcommand : GROUP ORDER $$8 grouplist",
"group : INTEGER",
"group : REAL",
"group : STRING",
"grouplist : group",
"grouplist : grouplist ',' group",
"allele : INTEGER",
"allele : REAL",
"allele : STRING",
"$$9 :",
"freqlist : allele ',' $$9 freqlist1",
"$$10 :",
"freqlist : freqlist allele ',' $$10 freqlist1",
"freqlist1 : rnum",
"freqlist1 : '*'",
"freqlist1 : freqlist1 ',' rnum",
"freqlist1 : freqlist1 ',' '*'",
"ibdlist : rnum",
"ibdlist : ibdlist ',' rnum",
"rnum : REAL",
"rnum : INTEGER",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 272 "param_parse.y"

static void set_analyze(char *p)
{
	int i;
	char *com[]={"AFFECTED","NULL","IBD","IBS",0};
		
	if(p) {
		i=0;
		while(com[i]) {
			if(!strcasecmp(com[i],p)) break;
			i++;
		}
		if(com[i]) analysis|=(1<<i);
		else yyerror("Invalid parameter to analyze statement");
	}
}

static void set_group_order( int gp)
{
	int i;
	
	if(!group_var || gp<0) return;
	for(i=0;i<group_ptr;i++) if(group_order[i]==gp)	{
		yyerror("Group repeated in order statement");
		return;
	}
	if(group_ptr>=group_var->n_levels) yyerror("Too many groups - internal error?");
	else group_order[group_ptr++]=gp;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "set_output_gen"
static void set_output_gen(char *file,char *link)
{
	int i;
	struct output_gen *p;

	if(link)	{
		for(i=0;i<n_links;i++) if(!strcasecmp(link,linkage[i].name)) break;
		if(i==n_links) return;
		i++;
	} else i=0;
	p=Output_Gen;
	if(!(Output_Gen=malloc(sizeof(struct output_gen)))) ABT_FUNC(MMsg);
	Output_Gen->next=p;
	Output_Gen->file=file;
	Output_Gen->link_group=i;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_ibd_list"
static struct IBD_List *add_ibd_list(double x,struct IBD_List *p)
{
	if(!p) {
		if(!(p=malloc(sizeof(struct IBD_List)))) ABT_FUNC(MMsg);
		p->idx=0;
		p->size=32;
		if(!(p->pos=malloc(sizeof(double)*p->size))) ABT_FUNC(MMsg);
	}
	if(p->size==p->idx) {
		p->size*=2;
		if(!(p->pos=realloc(p->pos,sizeof(double)*p->size))) ABT_FUNC(MMsg);
	}
	p->pos[p->idx++]=x;
	return p;
}

static void set_ibd_list(char *name,struct IBD_List *p)
{
	int i;
	
	if(name)	{
		for(i=0;i<n_links;i++) if(!strcasecmp(name,linkage[i].name)) break;
		if(i==n_links) {
			fprintf(stderr,"Warning: linkage group %s not found; IBD positions ignored\n",name);
			i= -1;
		}
	} else {
		if(n_links==1) i=0;
		else {
			i= -1;
			fprintf(stderr,"Warning: linkage group not specified when number of linkage groups >1; IBD positions ignored\n");
		}
	}
	if(i<0) {
		free(p->pos);
		free(p);
	} else {
		if(linkage[i].ibd_list)	{
			fprintf(stderr,"Warning: overwriting list of IBD positions for linkage group %s\n",linkage[i].name);
			free(linkage[i].ibd_list->pos);
			free(linkage[i].ibd_list);
		}
		linkage[i].ibd_list=p;
	}
}

static void set_tloci(int a,int b)
{
	if(a<0) {
		if(a== -1) min_tloci=max_tloci=b;
		else start_tloci=b;
	} else if(a<b) {
		min_tloci=a;
		max_tloci=b;
	} else {
		min_tloci=b;
		max_tloci=a;
	}
}

static struct Marker *check_marker(struct lk_variable *lkvar)
{
	struct Marker *mk=0;
	
	if(!lkvar) return 0;
	if(lkvar->type!=LK_TYPE_MARKER)
		yyerror("Attempting to set frequency of a non-marker");
	else mk=lkvar->var.marker;
	free(lkvar);
	return mk;
}

static void set_output(struct lk_variable *lkvar)
{
	int i,j,type;
	struct Variable *var;
	
	if(!lkvar) return;

	for(i=0;i<model.n_terms;i++)
	{
		type=model.term[i].vars[0].type;
		j=model.term[i].vars[0].var_index;
		if(type&ST_MARKER)
		{
			if(lkvar->type==LK_TYPE_MARKER && lkvar->var.marker==marker+j)
			{
				model.term[i].out_flag=1;
				return;
			}
		}
		else if(type&ST_TRAITLOCUS) continue;
		else
		{
			if(type&ST_CONSTANT) var=id_variable+j;
			else var=nonid_variable+j;
			if((lkvar->type==LK_TYPE_IDVAR || lkvar->type==LK_TYPE_NONIDVAR) && lkvar->var.var==var)
			{
				model.term[i].out_flag=1;
				return;
			}
		}
	}
}

static void set_map_range( char *name,double r1,double r2, int flag)
{
	int i;
	double t;
	static char *sexstr[2]={"female","male"};
	
	if(flag!= -1) sex_map=1;
	
	if(name)	{
		for(i=0;i<n_links;i++) if(!strcasecmp(name,linkage[i].name)) {
			if(r2<r1) {
				t=r2;
				r2=r1;
				r1=t;
			}
			if(flag== -1) {
				linkage[i].r1[0]=linkage[i].r1[1]=r1;
				linkage[i].r2[0]=linkage[i].r2[1]=r2;
				linkage[i].range_set[0]=linkage[i].range_set[1]=1;
				printf("Map range for linkage group '%s' set to %g-%gcM\n",name,r1,r2);
			} else {
				linkage[i].r1[flag]=r1;
				linkage[i].r2[flag]=r2;
				linkage[i].range_set[flag]=1;
				printf("Map range (%s) for linkage group '%s' set to %g-%gcM\n",sexstr[flag],name,r1,r2);
			}
			break;
		}
	} else {
		if(flag<0) {
			total_maplength[X_PAT]=r1;
			total_maplength[X_MAT]=r2;
			printf("Total (genome) map length set to (%g,%g)cM\n",r1,r2);
		} else {
			total_maplength[flag]=r1;
			printf("Total (genome) %s map length set to %gcM\n",sexstr[flag],r1);
		}
	}
}

static int find_group( char *p,int gp, int flag)
{
	int i,j;
	char *s;
	
	if(!group_var) return -1;
	j=group_var->n_levels;
	
	if(group_var->rec_flag==ST_STRING) {
		for(i=0;i<j;i++) if(!(strcasecmp(p,group_var->recode[i].string))) return i;
	} else {
		if(!flag) {
			gp=strtol(p,&s,10);
			if(!(*s)) for(i=0;i<j;i++) if(gp==group_var->recode[i].value) return i;
		} else for(i=0;i<j;i++) if(gp==group_var->recode[i].value) return i;
	}
	yyerror("Group not found\n");
	return -1;
}

static int find_allele( char *p, struct Marker *mk,int all, int flag)
{
	int i,j;
	char *s;
	
	if(!mk) return -1;
	j=extra_allele_flag?mk->n_alleles:mk->n_alleles-1;
	
	if(mk->rec_flag==ST_STRING) {
		for(i=0;i<j;i++) if(!(strcmp(p,mk->recode[i].string))) return i;
	} else {
		if(!flag) {
			all=strtol(p,&s,10);
			if(!(*s)) for(i=0;i<j;i++) if(all==mk->recode[i].value) return i;
		} else for(i=0;i<j;i++) if(all==mk->recode[i].value) return i;
	}
	return -1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "find_var"
static struct lk_variable *find_var( char *p, int idx, int flag)
{
	int i,j=0;
	struct lk_variable *lkv=0;
	
	if(flag) if(idx<1) {
		yyerror("Illegal array index");
		return 0;
	}
	for(i=0;i<n_markers;i++) if(!strcasecmp(marker[i].name,p)) {
		if(flag && idx!=marker[i].index) continue;
		j=LK_TYPE_MARKER;
		break;
	}
	if(!j) for(i=0;i<n_id_records;i++) if(!strcasecmp(id_variable[i].name,p)) {
		if(flag && idx!=id_variable[i].index) continue;
		j=LK_TYPE_IDVAR;
		break;
	}
	if(!j) for(i=0;i<n_nonid_records;i++) if(!strcasecmp(nonid_variable[i].name,p)) {
		if(flag && idx!=nonid_variable[i].index) continue;
		j=LK_TYPE_NONIDVAR;
		break;
	}
	if(!j && !flag) for(i=0;i<n_links;i++) if(!strcasecmp(linkage[i].name,p)) {
		j=LK_TYPE_LINK;
		break;
	}
	if(j) {
		if(!(lkv=malloc(sizeof(struct lk_variable)))) ABT_FUNC(MMsg);
		lkv->type=j;
		switch(j) {
		 case LK_TYPE_MARKER:
			lkv->var.marker=marker+i;
			break;
		 case LK_TYPE_LINK:
			lkv->var.link=linkage+i;
			break;
		 case LK_TYPE_IDVAR:
			lkv->var.var=id_variable+i;
			break;
		 case LK_TYPE_NONIDVAR:
			lkv->var.var=nonid_variable+i;
			break;
		}
	}
	return lkv;
}

static void set_freq(struct Marker *mk, double freq,int allele)
{
	static int fg,fg1;
	int i=0;
	
	group_counter++;
	if(mk) {
		if(group_var) {
			if(!group_ptr)	{
				if(!fg) yyerror("Genetic group order not set");
				fg=1;
				return;
			}
			if(group_counter>group_var->n_levels) {
				if(!fg1) yyerror("Too many frequencies specified (only 1 per genetic group)");
				fg1=1;
				return;
			}
			i=group_order[group_counter-1];
			if(i<0) return;
		}
		if(freq<0.0) {
			yyerror("Invalid (negative) frequency");
			return;
		}
		if(allele>=0 && freq==0.0)	{
			yyerror("Can not set frequency of observed allele to zero\n");
			return;
		}
		if(allele>=0) {
			mk->freq[i][allele]=freq;
		} else {
			if(extra_allele_flag) {
				yyerror("Cannot set frequency of unknown allele\n");
				return;
			}
			allele=mk->n_alleles-1;
			mk->freq[i][allele]+=freq;
		}
		mk->freq_set[i][allele]=start_flag;
		mk->count_flag[i]=c_flag;
	}
	fg1=0;
}

static void set_position(struct lk_variable *lkvar, double pos1, double pos2)
{
	if(lkvar) {
		if(lkvar->type!=LK_TYPE_MARKER) {
			yyerror("Attempting to set position of a non-marker");
			return;
		}
		lkvar->var.marker->pos[X_PAT]=pos1;
		lkvar->var.marker->pos[X_MAT]=pos2;
		lkvar->var.marker->pos_set=start_flag;
		free(lkvar);
	}
}

static int check_variance( double v)
{
	if(v<=0.0) {
		yyerror("Variance must be positive");
		return 1;
	}
	return 0;
}

void print_scan_err(char *fmt, ...)
{
	va_list args;
	
	va_start(args,fmt);
	vfprintf(stderr,fmt,args);
	va_end(args);
	if((++scan_error_n)>=max_scan_errors) abt(__FILE__,__LINE__,"Too many errors - aborting\n");
}

void print_scan_warn(char *fmt, ...)
{
	va_list args;
	
	if(scan_warn_n<max_scan_warnings)
	{
		va_start(args,fmt);
		vfprintf(stderr,fmt,args);
		va_end(args);
	}
	scan_warn_n++;
}

void yyerror(char *s)
{
     int i;
	
	print_scan_err("Line %d: %s\n%s\n",lineno,s,linebuf);
	if(scan_error_n<=max_scan_errors)
	{
		for(i=1;i<tokenpos;i++) putc('-',stderr);
		fputs("^\n",stderr);
	}
}

int yywrap(void)
{
	return 1;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "ReadParam"
int ReadParam(FILE *fptr)
{
	int i,j;
	void yy_cleanup(void);
	
	for(i=0;i<NUM_SYSTEM_VAR;i++) syst_var[i].flag=0;
	for(i=0;i<n_id_records;i++) if(id_variable[i].type&ST_GROUP) {
		group_var=id_variable+i;
		j=group_var->n_levels;
		if(!(group_order=malloc(sizeof(int)*j))) ABT_FUNC(MMsg);
		break;
	}
	yyin=fptr;
	yyparse();
	yy_cleanup();
	if(group_order) free(group_order);
	if(start_tloci<0) start_tloci=min_tloci;
	else if(start_tloci<min_tloci || start_tloci>max_tloci) {
		fprintf(stderr,"ReadParam(): Starting no. trait loci (%d) is outside set range (%d-%d)\n",start_tloci,min_tloci,max_tloci);
		scan_error_n++;
	}
	if(scan_error_n) return 1;
	return 0;
}
#line 1087 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 93 "param_parse.y"
{lineno1=lineno;}
break;
case 4:
#line 95 "param_parse.y"
{lineno1=lineno;}
break;
case 9:
#line 101 "param_parse.y"
{start_flag=2;}
break;
case 10:
#line 101 "param_parse.y"
{start_flag=1;}
break;
case 28:
#line 125 "param_parse.y"
{sample_from[0]=sample_from[1]=yyvsp[0].value;}
break;
case 29:
#line 126 "param_parse.y"
{sample_from[0]=yyvsp[-2].value; sample_from[1]=yyvsp[0].value;}
break;
case 30:
#line 127 "param_parse.y"
{sample_from[0]=sample_from[1]=yyvsp[0].value;}
break;
case 31:
#line 128 "param_parse.y"
{sample_from[0]=yyvsp[-2].value; sample_from[1]=yyvsp[0].value;}
break;
case 32:
#line 131 "param_parse.y"
{ syst_var[yyvsp[-1].value].data.value=yyvsp[0].value; syst_var[yyvsp[-1].value].flag=ST_INTEGER; }
break;
case 33:
#line 132 "param_parse.y"
{ syst_var[yyvsp[-1].value].data.rvalue=yyvsp[0].rvalue; syst_var[yyvsp[-1].value].flag=ST_REAL; }
break;
case 34:
#line 133 "param_parse.y"
{ free(yyvsp[-1].string); yyerror("Unrecognized system variable"); }
break;
case 35:
#line 136 "param_parse.y"
{ set_ibd_list(yyvsp[-2].string,yyvsp[0].rlist); free (yyvsp[-2].string);}
break;
case 36:
#line 137 "param_parse.y"
{ set_ibd_list(0,yyvsp[0].rlist); }
break;
case 37:
#line 140 "param_parse.y"
{set_analyze(yyvsp[0].string); free(yyvsp[0].string);}
break;
case 38:
#line 141 "param_parse.y"
{set_analyze("IBD");}
break;
case 41:
#line 148 "param_parse.y"
{analysis=0;}
break;
case 43:
#line 150 "param_parse.y"
{if(Seedfile) free(Seedfile); Seedfile=yyvsp[0].string;}
break;
case 44:
#line 151 "param_parse.y"
{if(Seedfile) free(Seedfile); Seedfile=yyvsp[-2].string; if(yyvsp[0].value) ranseed_set|=1; else ranseed_set&=~1;}
break;
case 45:
#line 152 "param_parse.y"
{if(Seedfile) free(Seedfile); Seedfile=yyvsp[0].string;}
break;
case 46:
#line 153 "param_parse.y"
{if(Seedfile) free(Seedfile); Seedfile=yyvsp[-2].string; if(yyvsp[0].value) ranseed_set|=1; else ranseed_set&=~1;}
break;
case 47:
#line 154 "param_parse.y"
{
			 if(yyvsp[0].value<0) yyerror("Seedvalue out of range");
			 else {
				 init_ranf(yyvsp[0].value);
				 ranseed_set|=2;
			 }
		 }
break;
case 48:
#line 162 "param_parse.y"
{set_map_range(yyvsp[-3].string,yyvsp[-2].rvalue,yyvsp[0].rvalue,-1); free(yyvsp[-3].string);}
break;
case 49:
#line 163 "param_parse.y"
{set_map_range(yyvsp[-3].string,yyvsp[-2].rvalue,yyvsp[0].rvalue,X_PAT); free(yyvsp[-3].string); }
break;
case 50:
#line 164 "param_parse.y"
{set_map_range(yyvsp[-3].string,yyvsp[-2].rvalue,yyvsp[0].rvalue,X_MAT); free(yyvsp[-3].string); }
break;
case 51:
#line 165 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,-1);}
break;
case 52:
#line 166 "param_parse.y"
{set_map_range(0,yyvsp[-2].rvalue,yyvsp[0].rvalue,-2);}
break;
case 53:
#line 167 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,X_PAT);}
break;
case 54:
#line 168 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,X_MAT);}
break;
case 55:
#line 169 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,-1);}
break;
case 56:
#line 170 "param_parse.y"
{set_map_range(0,yyvsp[-2].rvalue,yyvsp[0].rvalue,-2);}
break;
case 57:
#line 171 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,X_PAT);}
break;
case 58:
#line 172 "param_parse.y"
{set_map_range(0,yyvsp[0].rvalue,yyvsp[0].rvalue,X_MAT);}
break;
case 59:
#line 175 "param_parse.y"
{set_tloci(-1,yyvsp[0].value);}
break;
case 60:
#line 176 "param_parse.y"
{set_tloci(-2,yyvsp[0].value);}
break;
case 61:
#line 177 "param_parse.y"
{set_tloci(yyvsp[-2].value,yyvsp[0].value);}
break;
case 62:
#line 178 "param_parse.y"
{tloci_mean=yyvsp[0].rvalue; tloci_mean_set=1;}
break;
case 63:
#line 181 "param_parse.y"
{ num_iter=yyvsp[0].value; }
break;
case 64:
#line 183 "param_parse.y"
{sample_freq[0]=sample_freq[1]=yyvsp[0].value;}
break;
case 65:
#line 184 "param_parse.y"
{sample_freq[0]=yyvsp[-2].value; sample_freq[1]=yyvsp[0].value;}
break;
case 66:
#line 185 "param_parse.y"
{if(Freqfile) free(Freqfile); Freqfile=yyvsp[0].string;}
break;
case 67:
#line 186 "param_parse.y"
{sample_freq[0]=sample_freq[1]=yyvsp[0].value;}
break;
case 68:
#line 187 "param_parse.y"
{sample_freq[0]=yyvsp[-2].value; sample_freq[1]=yyvsp[0].value;}
break;
case 69:
#line 188 "param_parse.y"
{Output_Phen=yyvsp[0].string; }
break;
case 70:
#line 189 "param_parse.y"
{set_output_gen(yyvsp[0].string,0);}
break;
case 71:
#line 190 "param_parse.y"
{set_output_gen(yyvsp[-2].string,yyvsp[0].string); free(yyvsp[0].string); }
break;
case 73:
#line 192 "param_parse.y"
{output_type=yyvsp[0].value;}
break;
case 74:
#line 193 "param_parse.y"
{if(Outputfile) free(Outputfile); Outputfile=yyvsp[0].string;}
break;
case 75:
#line 194 "param_parse.y"
{if(Dumpfile) free(Dumpfile); Dumpfile=yyvsp[0].string;}
break;
case 76:
#line 195 "param_parse.y"
{dump_freq=yyvsp[0].value;}
break;
case 77:
#line 196 "param_parse.y"
{output_haplo=1;if(Haplofile) free(Haplofile); Haplofile=yyvsp[0].string;}
break;
case 78:
#line 197 "param_parse.y"
{output_haplo=1;}
break;
case 79:
#line 200 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) {res_var_set=start_flag; residual_var=yyvsp[0].rvalue;} }
break;
case 80:
#line 201 "param_parse.y"
{ use_student_t=0; }
break;
case 81:
#line 202 "param_parse.y"
{ use_student_t=1; }
break;
case 82:
#line 205 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) residual_var_limit=yyvsp[0].rvalue; }
break;
case 83:
#line 206 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) residual_var_limit=yyvsp[0].rvalue; }
break;
case 84:
#line 209 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) {add_var_set=start_flag; additive_var=yyvsp[0].rvalue;} }
break;
case 85:
#line 211 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) additive_var_limit=yyvsp[0].rvalue; }
break;
case 86:
#line 212 "param_parse.y"
{ if(!check_variance(yyvsp[0].rvalue)) additive_var_limit=yyvsp[0].rvalue; }
break;
case 87:
#line 215 "param_parse.y"
{ grand_mean_set=start_flag; grand_mean=yyvsp[0].rvalue; }
break;
case 88:
#line 217 "param_parse.y"
{ set_position(yyvsp[-1].lk_var,yyvsp[0].rvalue,yyvsp[0].rvalue); }
break;
case 89:
#line 218 "param_parse.y"
{ sex_map=1; set_position(yyvsp[-3].lk_var,yyvsp[-2].rvalue,yyvsp[0].rvalue); }
break;
case 90:
#line 221 "param_parse.y"
{c_flag=0;}
break;
case 92:
#line 222 "param_parse.y"
{c_flag=1;}
break;
case 94:
#line 223 "param_parse.y"
{c_flag=1;}
break;
case 96:
#line 226 "param_parse.y"
{ yyval.lk_var=find_var(yyvsp[0].string,0,0); free(yyvsp[0].string);}
break;
case 97:
#line 227 "param_parse.y"
{ yyval.lk_var=find_var(yyvsp[-3].string,yyvsp[-1].value,1); free(yyvsp[-3].string);}
break;
case 98:
#line 230 "param_parse.y"
{ if(yyvsp[0].lk_var) {set_output(yyvsp[0].lk_var); free(yyvsp[0].lk_var);} }
break;
case 99:
#line 231 "param_parse.y"
{ if(yyvsp[0].lk_var) {set_output(yyvsp[0].lk_var); free(yyvsp[0].lk_var);} }
break;
case 100:
#line 234 "param_parse.y"
{ freq_marker=check_marker(yyvsp[0].lk_var); }
break;
case 101:
#line 236 "param_parse.y"
{group_ptr=0;}
break;
case 102:
#line 236 "param_parse.y"
{if(group_var && group_ptr<group_var->n_levels) print_scan_err("Line %d: Too few groups in order statement\n",lineno1);}
break;
case 103:
#line 239 "param_parse.y"
{ yyval.value=find_group(yytext,yyvsp[0].value,1); }
break;
case 104:
#line 240 "param_parse.y"
{ yyval.value=find_group(yytext,0,0); }
break;
case 105:
#line 241 "param_parse.y"
{ yyval.value=find_group(yyvsp[0].string,0,0); free(yyvsp[0].string);}
break;
case 106:
#line 244 "param_parse.y"
{set_group_order(yyvsp[0].value);}
break;
case 107:
#line 245 "param_parse.y"
{set_group_order(yyvsp[0].value);}
break;
case 108:
#line 248 "param_parse.y"
{ yyval.value=find_allele(yytext,freq_marker,yyvsp[0].value,1); }
break;
case 109:
#line 249 "param_parse.y"
{ yyval.value=find_allele(yytext,freq_marker,0,0); }
break;
case 110:
#line 250 "param_parse.y"
{ yyval.value=find_allele(yyvsp[0].string,freq_marker,0,0); free(yyvsp[0].string); }
break;
case 111:
#line 253 "param_parse.y"
{group_counter=0; freq_allele=yyvsp[-1].value;}
break;
case 112:
#line 253 "param_parse.y"
{if(freq_marker && group_var && group_counter<group_var->n_levels) print_scan_err("Line %d: Too few frequencies specified\n",lineno);}
break;
case 113:
#line 254 "param_parse.y"
{group_counter=0; freq_allele=yyvsp[-1].value;}
break;
case 114:
#line 254 "param_parse.y"
{if(freq_marker && group_var && group_counter<group_var->n_levels) print_scan_err("Line %d: Too few frequencies specified\n",lineno);}
break;
case 115:
#line 257 "param_parse.y"
{set_freq(freq_marker,yyvsp[0].rvalue,freq_allele);}
break;
case 116:
#line 258 "param_parse.y"
{group_counter++;}
break;
case 117:
#line 259 "param_parse.y"
{set_freq(freq_marker,yyvsp[0].rvalue,freq_allele);}
break;
case 118:
#line 260 "param_parse.y"
{group_counter++;}
break;
case 119:
#line 263 "param_parse.y"
{ yyval.rlist=add_ibd_list(yyvsp[0].rvalue,0); }
break;
case 120:
#line 264 "param_parse.y"
{ yyval.rlist=add_ibd_list(yyvsp[0].rvalue,yyvsp[-2].rlist); }
break;
case 122:
#line 268 "param_parse.y"
{ yyval.rvalue=(double)yyvsp[0].value; }
break;
#line 1652 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
