/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                   Simon Heath - MSKCC                                    *
 *                                                                          *
 *                       March 2000                                         *
 *                                                                          *
 * sample_all_segs.c:                                                       *
 *                                                                          *
 * Sample all segregation indicators for all loci simultaneously            *
 * conditional on sampled *un*ordered genotypes                             *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/times.h>
#include <time.h>
#include <sys/wait.h>

#include "ranlib.h"
#include "version.h"
#include "utils.h"
#include "loki.h"
#include "loki_peel.h"
#include "loki_ibd.h"
#include "sample_segs.h"

static struct sg_off **diag;
static struct node *node;

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "order_family"
static void order_family(struct family *f,int *fg,const int locus)
{
	int i,i1,j[2],j1[2],k,k1,k2,ids,idd,mt[2],pt[2],kt[2];
	int nkids,*kids,kid;
	lk_ulong a[2],c[2];
	
	ids=f->ids-1;
	idd=f->idd-1;
	nkids=f->nkids;
	kids=f->kids;
	mt[0]=id_array[idd].allele[0];
	mt[1]=id_array[idd].allele[1];
	pt[0]=id_array[ids].allele[0];
	pt[1]=id_array[ids].allele[1];
	if(mt[0]==mt[1]) fg[idd]|=16;
	if(pt[0]==pt[1]) fg[ids]|=16;
	for(k=0;k<nkids;k++) {
		kid=kids[k];
		k1=(fg[kid]&4)?1:2;
		kt[0]=id_array[kid].allele[0];
		kt[1]=id_array[kid].allele[1];
		if(kt[0]==kt[1]) {
			fg[kid]|=16;
		}
		i1=0;
		for(k2=0;k2<2;k2++) j[k2]=j1[k2]=0;
		if(locus<0) for(k2=0;k2<k1;k2++) {
			if(kt[k2]==mt[0]) j[k2]|=1;
			if(kt[k2]==mt[1]) j[k2]|=2;
			if(j[k2]) {
				if(kt[k2^1]==pt[0]) j1[k2]|=1;
				if(kt[k2^1]==pt[1]) j1[k2]|=2;
			}
			if(!(j[k2]&&j1[k2])) {
				if(!k2) ABT_FUNC("Oook!\n");
				fg[kid]|=4;
			} else i1++;
		} else for(k2=0;k2<k1;k2++) {
			c[0]=req_set[k2][locus][kid];
			a[0]=1<<(kt[k2]-1);
			c[1]=req_set[k2^1][locus][kid];
			a[1]=1<<(kt[k2^1]-1);
			for(i=0;i<2;i++) if(a[i]&c[i]) a[i]=c[i];
			if(a[0]&(1<<(mt[0]-1))) j[k2]|=1;
			if(a[0]&(1<<(mt[1]-1))) j[k2]|=2;
			if(j[k2]) {
				if(a[1]&(1<<(pt[0]-1))) j1[k2]|=1;
				if(a[1]&(1<<(pt[1]-1))) j1[k2]|=2;
			}
			if(!(j[k2]&&j1[k2])) {
				fg[kid]|=4;
			} else i1++;
		}
		if(!i1) {
			ABT_FUNC("Internal error - bad sampled genotype\n");
		}
		if(i1==2) {
			if(j[0]==3 && j[1]==3) fg[kid]|=MAT_SEG_IRR;
			else if(fg[idd]&4) {
				if(j[0]==1 && j[1]==1) fg[kid]|=MAT_SEG_FIXED_M;
				else if(j[0]==2 && j[1]==2) fg[kid]|=MAT_SEG_FIXED_P;
			}
			if(j1[0]==3 && j1[1]==3) fg[kid]|=PAT_SEG_IRR;
			else if(fg[ids]&4) {
				if(j1[0]==1 && j1[1]==1) fg[kid]|=PAT_SEG_FIXED_M;
				else if(j1[0]==2 && j1[1]==2) fg[kid]|=PAT_SEG_FIXED_P;
			}
		} else {
			if(j[0]&&j1[0]) k2=0;
			else {
				k2=1;
				id_array[kid].allele[0]=kt[1];
				id_array[kid].allele[1]=kt[0];
			}
			if(j[k2]==3) fg[kid]|=MAT_SEG_IRR;
			else if(fg[idd]&4) {
				if(j[k2]==1) fg[kid]|=MAT_SEG_FIXED_M;
				else fg[kid]|=MAT_SEG_FIXED_P;
			}
			if(j1[k2]==3) fg[kid]|=64;
			else if(fg[ids]&4) {
				if(j1[k2]==1) fg[kid]|=PAT_SEG_FIXED_M;
				else fg[kid]|=PAT_SEG_FIXED_P;
			}
		}
	}
}

void print_node(FILE *fptr,const int x)
{
	if(node[x].type==NODE_ORDER) fputs("ORDER",fptr);
	else if(node[x].type==NODE_MAT_SEG) fputs("MAT_SEG",fptr);
	else if(node[x].type==NODE_PAT_SEG) fputs("PAT_SEG",fptr);
	else fputs("**BAD NODE**",fptr);
	fprintf(fptr,"(%d: %d %d)",x,node[x].id+1,node[x].locus);
}

void add_off(const int x,const int y) 
{
	print_node(stdout,x);
	fputc(' ',stdout);
	print_node(stdout,y);
	fputc('\n',stdout);
}

static int assemble_matrix(int **flags,int **index,int *perm,int nl)
{
	int i,k,k0,k1,k2,x,*fg,*idx,len,locus,ids,idd;
	
	for(len=x=0;x<nl;x++) {
		fg=flags[x];
		idx=index[x];
		locus=perm[x];
		for(i=0;i<ped_size;i++) if(!(fg[i]&8)) {
			if(locus<0) k=tlocus[-1-locus].gt[i];
			else k=marker[locus].gt[i]-1;
			k1=(int)(.5*(1.0+sqrt(1.0+8.0*(double)k)));
			id_array[i].allele[0]=k1;
			id_array[i].allele[1]=k+1-k1*(k1-1)/2;
			printf("%d %x (%d %d)\n",i+1,fg[i],id_array[i].allele[0],id_array[i].allele[1]);
			k=k0=idx[i];
			if(fg[i]&NODE_ORDER) k++;
			k1=k;
			if(fg[i]&NODE_MAT_SEG) k++;
			k2=k;
			ids=idd=0;
			if(fg[i]&NODE_ORDER) {
				fputs("[ORDER]\n",stdout);
				if(fg[i]&NODE_MAT_SEG) add_off(k1,k0);
				if(fg[i]&NODE_PAT_SEG) add_off(k2,k0);
			}
 			if(fg[i]&NODE_MAT_SEG) {
				fputs("[MAT_SEG]\n",stdout);
				idd=id_array[i].dam;
				if(fg[idd-1]&NODE_ORDER) {
					if(fg[i]&NODE_ORDER) add_off(k0,idx[idd-1]);
					add_off(k1,idx[idd-1]);
				} else idd=0;
				if(fg[i]&NODE_PAT_SEG) {
					add_off(k2,k1);
				}
			}
 			if(fg[i]&NODE_PAT_SEG) {
				fputs("[PAT_SEG]\n",stdout);
				ids=id_array[i].sire;
				if(fg[ids-1]&NODE_ORDER) {
					if(fg[i]&NODE_ORDER)	add_off(k0,idx[ids-1]);
					add_off(k2,idx[ids-1]);
				} else ids=0;
			}
			if(ids&&idd) {
				if(ids<idd) add_off(idx[idd-1],idx[ids-1]);
				else add_off(idx[ids-1],idx[idd-1]);
			}
		}
	}
	return len;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "sample_all_segs"
void sample_all_segs(const int link)
{
	int i,j,k,k1,x,kid,nfam,id,comp,locus,idd,ids,max_fam,nl;
	int *kd=0,nk,max_nk=0,nxt[6],*perm,**flags,**index,*fg;
	struct family *fam;
	
	k=linkage[link].n_markers+n_tloci;
	if(!k) return;
	if(!(perm=malloc(sizeof(int)*k))) ABT_FUNC(MMsg);
	get_locuslist(perm,link,&nl);
	gnu_qsort(perm,(size_t)nl,(size_t)sizeof(int),cmp_loci_pos);
	if(!nl) {
		free(perm);
		return;
	}
	if(!(flags=malloc(sizeof(void *)*nl*2))) ABT_FUNC(MMsg);
	if(!(flags[0]=malloc(sizeof(void *)*(1+ped_size*nl*2)))) ABT_FUNC(MMsg);
	for(i=1;i<nl*2;i++) flags[i]=flags[i-1]+ped_size;
	index=flags+nl;
	max_fam=64;
	if(!(fam=malloc(sizeof(struct family)*max_fam))) ABT_FUNC(MMsg);
	for(x=0;x<nl;x++) {
		locus=perm[x];
		fg=flags[x];
		for(id=comp=0;comp<n_comp;comp++) {
			nfam=0;
			for(j=0;j<comp_size[comp];j++) {
				i=id+j;
				if(id_array[i].pruned_flag[locus<0?n_markers:locus]) {
					/* Pruned */
					fg[i]=8;
					continue;
				} else if(id_array[i].sire && !(fg[id_array[i].sire-1]&8)) {
					/* Non-founder */
					fg[i]=0;
				} else {
					/* Founder */
					fg[i]=1;
				}
				if(locus<0) k=tlocus[-1-locus].gt[i];
				else k=marker[locus].gt[i]-1;
				k1=(int)(.5*(1.0+sqrt(1.0+8.0*(double)k)));
				id_array[i].allele[0]=k1;
				id_array[i].allele[1]=k+1-k1*(k1-1)/2;
			}
			/* Count families */
			for(nk=j=0;j<comp_size[comp];j++) {
				i=id+j;
				if(fg[i]) continue;
				ids=id_array[i].sire;
				idd=id_array[i].dam;
				if(nfam>=max_fam) {
					max_fam*=2;
					if(!(fam=realloc(fam,sizeof(struct family)*max_fam))) ABT_FUNC(MMsg);
				}
				fam[nfam].nkids=0;
				for(k=0;k<id_array[ids-1].nkids;k++) {
					kid=id_array[ids-1].kids[k];
					if(!fg[kid] && id_array[kid].sire==ids && id_array[kid].dam==idd) {
						fg[kid]|=2;
						fam[nfam].nkids++;
					}
				}
				nk+=fam[nfam].nkids;
				fam[nfam].ids=ids;
				fam[nfam++].idd=idd;
			}
			printf("Comp %d, No. families = %d\n",comp+1,nfam);
			if(nk && nk>=max_nk) {
				max_nk=nk;
				if(!(kd=realloc(kd,sizeof(int)*max_nk))) ABT_FUNC(MMsg);
			}
			for(nk=j=0;j<nfam;j++) {
				ids=fam[j].ids;
				idd=fam[j].idd;
				fam[j].kids=kd+nk;
				for(k=0;k<id_array[ids-1].nkids;k++) {
					kid=id_array[ids-1].kids[k];
					if((fg[kid]&2) && id_array[kid].sire==ids && id_array[kid].dam==idd) {
						kd[nk++]=kid;
					}
				}
				order_family(fam+j,fg,locus);
			}
			for(j=0;j<6;j++) nxt[j]=0;
			for(j=0;j<comp_size[comp];j++) {
				i=id+j;
				if(fg[i]&8) {
					nxt[0]++;
					continue;
				}
				if(fg[i]&20) nxt[1]++;
				else nxt[2]++;
				if(fg[i]&1) continue;
				if(fg[i]&32) nxt[3]++;
				else if(fg[i]&0x180) nxt[4]++;
				else nxt[5]++;
				if(fg[i]&64) nxt[3]++;
				else if(fg[i]&0x600) nxt[4]++;
				else nxt[5]++;
			}
			for(j=0;j<6;j++) printf("%d ",nxt[j]);
			printf("\n");
			id+=comp_size[comp];
		}
	}
	/* Count nodes */
	for(j=x=0;x<nl;x++) {
		fg=flags[x];
		for(i=0;i<ped_size;i++) {
			if(fg[i]&8) {
				index[x][i]=-1;
				continue;
			}
			index[x][i]=j;
			/* If not a founder */
			if(!(fg[i]&1)) {
				/* is the Maternal SI needed? */
				if(!(fg[i]&(MAT_SEG_IRR|MAT_SEG_FIXED_M|MAT_SEG_FIXED_P))) {
					j++;
					fg[i]|=NODE_MAT_SEG;
				}
				/* is the Paternal SI needed? */
				if(!(fg[i]&(PAT_SEG_IRR|PAT_SEG_FIXED_M|PAT_SEG_FIXED_P))) {
					j++;
					fg[i]|=NODE_PAT_SEG;
				}
			}
			/* Is order fixed ? */
			if(!(fg[i]&20)) {
				j++;
				fg[i]|=NODE_ORDER;
			}
		}
	}
	if(j) {
		index[nl-1][ped_size]=j;
		if(!(diag=malloc(sizeof(void *)*j))) ABT_FUNC(MMsg);
		if(!(node=malloc(sizeof(struct node)*j))) ABT_FUNC(MMsg);
		for(i=0;i<j;i++) diag[i]=0;
		for(j=x=0;x<nl;x++) {
			fg=flags[x];
			for(i=0;i<ped_size;i++) if(!(fg[i]&8)) {
				if(fg[i]&NODE_ORDER) {
					node[j].type=NODE_ORDER;
					node[j].id=i;
					node[j++].locus=x;
				}
				if(fg[i]&NODE_MAT_SEG) {
					node[j].type=NODE_MAT_SEG;
					node[j].id=i;
					node[j++].locus=x;
				}
				if(fg[i]&NODE_PAT_SEG) {
					node[j].type=NODE_PAT_SEG;
					node[j].id=i;
					node[j++].locus=x;
				}
			}
		}
		k=assemble_matrix(flags,index,perm,nl);
		printf("k = %d\n",k);
		free(node);
		free(diag);
	}
	free(flags[0]);
	free(flags);
	free(fam);
	free(perm);
	if(kd) free(kd);
	exit(0);
}
