/****************************************************************************
 *                                                                          *
 *     Loki - Programs for genetic analysis of complex traits using MCMC    *
 *                                                                          *
 *                   Simon Heath - MSKCC                                    *
 *                                                                          *
 *                       August 2000                                        *
 *                                                                          *
 * sample_effects.c:                                                        *
 *                                                                          *
 * Sampling routine for all effects                                         *
 *                                                                          *
 ****************************************************************************/

#include <config.h>

#include <math.h>
#include <stdio.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "ranlib.h"
#include "version.h"
#include "utils.h"
#include "sparse.h"
#include "loki.h"
#include "amdbar.h"
#include "loki_tlmoves.h"
#include "sample_rand.h"
#include "sample_effects.h"

static int n_var,n_lev,cov_start,mk_start,rand_start,poly_start;
static int bsize=1024,*order,*order1,XX_size,entry_size,*zero;
static double *B;
static struct Diag *XX;
static struct Off *freelist;
struct entry {
	int pos;
	double val;
};
static struct entry *entry;

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "init_sample_effects"
static struct Off *alloc_new_nodes(int n)
{
	struct Off *p;
	static int count,count1;
	int i;
	
	if(!n) ABT_FUNC("Internal error - called with zero argument\n");
	if(!(p=malloc(sizeof(struct Off)*n))) ABT_FUNC(MMsg);
	RemBlock=AddRemem(p,RemBlock);
	/* Link blocks together */
	for(i=0;i<n-1;i++) p[i].Next=p+i+1;
	p[n-1].Next=0;
	count++;
	count1+=n;
	return p;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "get_node"
static struct Off *get_node(void)
{
	struct Off *p;
	
	if(!freelist) freelist=alloc_new_nodes(bsize);
	p=freelist;
	freelist=p->Next;
	return p;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "add_XX_entry"
void add_XX_entry(int x,int y,double z)
{
	struct Off *p,**pp;
	int t;
	
	if(y>=x) {
		t=x;
		x=y;
		y=t;
	}
	pp=&XX[x].First;
	p=*pp;
	while(p) {
		if(p->col>=y) break;
		pp=&p->Next;
		p=p->Next;
	}
	if(p && p->col==y) p->val+=z;
	else {
		p=get_node();
		p->col=y;
		p->val=z;
		p->Next=*pp;
		*pp=p;
		XX[x].count++;
		XX[y].count1++;
	}
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "init_sample_effects"
void init_sample_effects(void)
{
	int i,j,k,k1,k2,k3,type,rec,nrec,n_lev1,n_var1,comp;
	int *pe,*iw,*degree,*nv,*next,*head,*w,*len;
	int ncmpa,iovflo,pfree;
	struct id_data *data;
	struct SparseMatRec *AI;
	struct Off *p,*p1;
	
	n_lev=n_var=0;
	poly_start=mk_start=rand_start=cov_start=-1;
	/* Count variables and levels (ignoring QTL's and mean) */
	for(i=k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_MARKER) {
			if(mk_start<0) mk_start=n_lev;
			n_lev+=model.term[k].df;
			n_var++;
		}
	}
	if(polygenic_flag) for(k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_ID) {
			if(poly_start<0) poly_start=n_lev;
			n_lev+=ped_size;
			n_var++;
		}
	}
	if(n_random) for(k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_RANDOM) {
			if(rand_start<0) rand_start=n_lev;
			n_lev+=model.term[k].df;
			n_var++;
		}
	}
	for(k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&(ST_MARKER|ST_TRAITLOCUS|ST_ID|ST_RANDOM)) continue;
		if(cov_start<0) cov_start=n_lev;
		n_lev+=model.term[k].df;
		n_var++;
	}
	if(!n_lev) return;
	/* Allocate X'X Matrix */
	/* Make it a bit bigger so we can use it later */
	XX_size=(int)(1.1*(double)(n_lev+2+max_tloci*2));
	if(!(XX=malloc(sizeof(struct Diag)*XX_size))) ABT_FUNC(MMsg);
	if(!(zero=malloc(sizeof(int)*XX_size))) ABT_FUNC(MMsg);
	if(!(B=malloc(sizeof(double)*XX_size))) ABT_FUNC(MMsg);
	for(i=0;i<n_lev;i++) {
		XX[i].First=0;
		XX[i].count=XX[i].count1=0;
	}
	entry_size=(int)(1.1*(double)n_var+2+max_tloci);
	if(!(entry=malloc(sizeof(struct entry)*entry_size))) ABT_FUNC(MMsg);
	freelist=alloc_new_nodes(bsize);
	/* Assemble the X'X matrix with non-zero elements.  Need to do this so we can
	   call amdbar() and get an order for factorizing the matrix */
	for(i=0;i<ped_size;i++) if(id_array[i].res) {
		nrec=id_array[i].n_rec;
		if(!nrec) nrec=1;
		for(rec=0;rec<nrec;rec++) {
			/* Set up contribution vector */
			n_lev1=n_var1=0;
			/* Candidate genes */
			for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_MARKER) {
					k1=model.term[k].vars[0].var_index;
					k2=marker[k1].gt[i]-1;
					if(k2<0) ABT_FUNC("Internal error - candidate gene with unsampled genotype\n");
					if(k2--)	{
						entry[n_var1].pos=n_lev1+k2;
					} else entry[n_var1].pos=-1;
					n_var1++;
					n_lev1+=model.term[k].df;
				}
			}
			/* Polygenic effect */
			if(polygenic_flag) for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_ID) {
					entry[n_var1].pos=n_lev1+i; 
					n_lev1+=ped_size;
					n_var1++;
				}
			}
			/* Additional random effects */
			if(n_random) for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_RANDOM) {
					k1=model.term[k].vars[0].var_index;
					data=0;
					if(type&ST_CONSTANT)	{
						if(id_array[i].data) data=id_array[i].data+k1;
					} else if(id_array[i].data1) data=id_array[i].data1[rec]+k1;
					if(type&ST_FACTOR) { /* Discrete factors */
						k2=(int)data->data.value-1;
						if(k2<0 || k2>model.term[k].df) ABT_FUNC("Internal error - illegal factor value\n");
						entry[n_var1].pos=n_lev1+k2;
					} else ABT_FUNC("Illegal Random type\n");
					n_lev1+=model.term[k].df;
					n_var1++;
				}
			}
			/* Other Covariates */
			for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&(ST_ID|ST_TRAITLOCUS|ST_RANDOM|ST_MARKER)) continue;
				k1=model.term[k].vars[0].var_index;
				data=0;
				if(type&ST_CONSTANT)	{
					if(id_array[i].data) data=id_array[i].data+k1;
				} else if(id_array[i].data1) data=id_array[i].data1[rec]+k1;
				if(type&ST_FACTOR) { /* Discrete factors */
					k2=(int)data->data.value-1;
					if(k2<0 || k2>model.term[k].df) ABT_FUNC("Internal error - illegal factor value\n");
					if(k2--)	entry[n_var1].pos=n_lev1+k2;
					else entry[n_var1].pos=-1;
				} else { /* Continuous factors */
					entry[n_var1].pos=n_lev1;
				}
				n_var1++;
				n_lev1+=model.term[k].df;
			}
			if(n_var1!=n_var || n_lev1!=n_lev) {
				fprintf(stderr,"%d %d  %d %d\n",n_var,n_var1,n_lev,n_lev1);
				ABT_FUNC("Internal error - level mismatch\n");
			}
			/* Add contributions to X'X */
			k=model.var.var_index;
			for(k=0;k<n_var;k++) {
				k1=entry[k].pos;
				if(k1>=0) {
					XX[k1].val=1.0;
					for(k2=k+1;k2<n_var;k2++) if((k3=entry[k2].pos)>=0) {
 						add_XX_entry(k3,k1,1.0);
					}
				}
			}
		}
	}
	if(polygenic_flag) {
		/* Add A^-1 Matrix */
		j=poly_start;
		for(comp=0;comp<n_comp;comp++) {
			AI=AIMatrix[comp];
			for(i=0;i<comp_size[comp];i++) {
				k=j+i;
				for(k1=AI[i].x;k1<AI[i+1].x;k1++) {
					k2=AI[k1].x+j;
					add_XX_entry(k,k2,1.0);
				}
			}
			j+=i;
		}
	}
	/* Assemble matrix for amdbar */
	/* Count off-diagonal elements */
	for(j=i=0;i<n_lev;i++) j+=XX[i].count+XX[i].count1;
	if(j+n_lev>(int)(1.0+(double)j*1.2)) j+=n_lev;
	else j=(int)(1.0+(double)j*1.2);
	if(!(order=malloc(sizeof(int)*2*n_lev))) ABT_FUNC(MMsg);
	if(!(pe=malloc(sizeof(int)*(7*n_lev+j)))) ABT_FUNC(MMsg);
	order1=order+n_lev;
	degree=pe+n_lev;
	nv=degree+n_lev;
	next=nv+n_lev;
	head=next+n_lev;
	w=head+n_lev;
	len=w+n_lev;
	iw=len+n_lev;
	for(k3=i=0;i<n_lev;i++) {
		if((len[i]=XX[i].count+XX[i].count1)) {
			pe[i]=k3+1;
			p=p1=XX[i].First;
			while(p) {
				iw[k3++]=p->col+1;
				iw[head[p->col]++]=i+1;
				p1=p;
				p=p->Next;
			}
			if(p1) {
				p1->Next=freelist;
				freelist=XX[i].First;
			}
			head[i]=k3;
			k3+=XX[i].count1;
		} else pe[i]=0;
	}
	pfree=1+k3;
	/* Internal checks */
	for(i=0;i<n_lev;i++) {
		k=pe[i];
		if(k<0 || k>pfree-len[i]) ABT_FUNC("Illegal index\n");
		for(k1=0;k1<len[i];k1++) {
			if(iw[k1+k-1]<=0 || iw[k1+k-1]>n_lev) ABT_FUNC("Illegal index\n");
			if(k1 && iw[k+k1-1]<iw[k+k1-2]) ABT_FUNC("Entries not in order\n");
		}
	}
	/* Get factorization order */
	amdbar_(&n_lev,pe,iw,len,&j,&pfree,nv,next,order,head,order1,degree,&ncmpa,w,&iovflo);
	free(pe);
}

void free_sample_effects(void)
{
	if(XX) free(XX);
	if(zero) free(zero);
	if(B) free(B);
	if(entry) free(entry);
	if(order) free(order);
	XX=0;
	B=0;
	entry=0;
	order=0;
	zero=0;
	XX_size=entry_size=0;
}

#ifdef FUNC_NAME
#undef FUNC_NAME
#endif
#define FUNC_NAME "sample_effects"
double sample_effects(void)
{
	int i,j,k,k1,k2,k3,mtype,type,idx,rec,nrec,n_qt,n_qtlev,n_var1,n_lev1,comp,nn;
	int b_var,b_lev,t_var,t_lev,qt_start;
	double y,z,lk,det,ss,ssn,wt;
	struct id_data *data;
	struct SparseMatRec *AI;
	struct Off *p,*p1;

	/* Get Ordering of equations */
	if(!order) init_sample_effects();
	n_qt=n_qtlev=0;
	/* Count QTL's */
	for(j=0;j<n_tloci;j++) if(tlocus[j].flag) {
		k2=tlocus[j].n_alleles;
		n_qt++;
		n_qtlev+=k2*(k2+1)/2-1;
	}
	/* Add one for data */
	b_var=n_qt+1;
	b_lev=n_qtlev+1;
	/* Add one for the grand mean if not fixed */
	if(grand_mean_set!=1) {
		b_var++;
		b_lev++;
	}
	qt_start=b_lev-n_qtlev;
	t_var=b_var+n_var;
	t_lev=b_lev+n_lev;
	if(!(t_var&&t_lev)) return 0.0;
	/* Check size of X'X Matrix */
	if(t_lev>XX_size) {
		XX_size=(int)((double)t_lev*1.1);
		if(!(XX=realloc(XX,sizeof(struct Diag)*XX_size))) ABT_FUNC(MMsg);
		if(!(zero=realloc(zero,sizeof(int)*XX_size))) ABT_FUNC(MMsg);
		if(!(B=realloc(B,sizeof(double)*XX_size))) ABT_FUNC(MMsg);
	}
	for(i=0;i<t_lev;i++) {
		XX[i].First=0;
		XX[i].count=XX[i].count1=0;
		XX[i].val=0;
	}
	if(t_var>entry_size) {
		entry_size=(int)(1.1*(double)t_var);
		if(!(entry=realloc(entry,sizeof(struct entry)*entry_size))) ABT_FUNC(MMsg);
	}
	/* Assemble the X'X matrix */
	mtype=model.var.type;
	idx=model.var.var_index;
	lk=0.0;
	for(i=0;i<ped_size;i++) if(id_array[i].res) {
		nrec=id_array[i].n_rec;
		if(!nrec) nrec=1;
		for(rec=0;rec<nrec;rec++) {
			wt=use_student_t?id_array[i].vv[rec]:1.0;
			/* Load raw data value into y */
			y=0.0;
			if(mtype&ST_CONSTANT) {
				data=id_array[i].data+idx;
				if(data->flag&2) y=id_array[i].cens[0];
			} else {
				data=id_array[i].data1[rec]+idx;
				if(data->flag&2) y=id_array[i].cens[rec];
			}
			if(data->flag&ST_INTTYPE) y+=(double)data->data.value;
			else y+=data->data.rvalue;
			/* Set up contribution vector */
			n_lev1=n_var1=1;
			/* ...mean */
			if(grand_mean_set!=1) {
				entry[n_var1].val=1.0;
				entry[n_var1++].pos=n_lev1++;
			} else y-=grand_mean;
			/* ...QTL's */
			if(n_qt) {
				for(j=0;j<n_tloci;j++) if(tlocus[j].flag)	{
					k2=tlocus[j].gt[i]-1;
					k1=tlocus[j].n_alleles;
					if(k2--)	{
						entry[n_var1].val=1.0;
						entry[n_var1].pos=n_lev1+k2;
					} else entry[n_var1].pos= -1;
					n_lev1+=k1*(k1+1)/2-1;
					n_var1++;
				}
			}
			/* Candidate genes */
			if(mk_start>=0) for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_MARKER) { 
					k1=model.term[k].vars[0].var_index;
					k2=marker[k1].gt[i]-1;
					if(k2<0) ABT_FUNC("Internal error - candidate gene with unsampled genotype\n");
					if(k2--)	{
						entry[n_var1].val=1.0;
						entry[n_var1].pos=n_lev1+k2;
					} else entry[n_var1].pos=-1;
					n_var1++;
					n_lev1+=model.term[k].df;
				}
			}
			/* Polygenic effect */
			if(polygenic_flag) for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_ID) {
					entry[n_var1].val=1.0;
					entry[n_var1++].pos=n_lev1+i; /* Polygenic effect */
					n_lev1+=ped_size;
				}
			}
			/* Additional random effects */
			if(n_random) for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&ST_RANDOM) {
					k1=model.term[k].vars[0].var_index;
					data=0;
					if(type&ST_CONSTANT)	{
						if(id_array[i].data) data=id_array[i].data+k1;
					} else if(id_array[i].data1) data=id_array[i].data1[rec]+k1;
					if(type&ST_FACTOR) { /* Discrete factors */
						k2=(int)data->data.value-1;
						if(k2<0 || k2>model.term[k].df) ABT_FUNC("Internal error - illegal factor value\n");
						entry[n_var1].pos=n_lev1+k2;
						entry[n_var1].val=1.0;
					} else ABT_FUNC("Illegal Random type\n");
					n_lev1+=model.term[k].df;
					n_var1++;
				}
			}
			/* ...other effects */
			for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&(ST_TRAITLOCUS|ST_ID|ST_MARKER|ST_RANDOM)) continue; /* Already done */
				k1=model.term[k].vars[0].var_index;
				data=0;
				if(type&ST_CONSTANT)	{
					if(id_array[i].data) data=id_array[i].data+k1;
				} else if(id_array[i].data1) data=id_array[i].data1[rec]+k1;
				if(type&ST_FACTOR) { /* Discrete factors */
					k2=(int)data->data.value-1;
					if(k2<0 || k2>model.term[k].df) ABT_FUNC("Internal error - illegal factor value\n");
					if(k2--)	{
						entry[n_var1].pos=n_lev1+k2;
						entry[n_var1].val=1.0;
					} else entry[n_var1].pos=-1;
				} else { /* Continuous factors */
					if(data->flag&ST_INTTYPE) z=(double)data->data.value;
					else z=data->data.rvalue;
					entry[n_var1].pos=n_lev1;
					entry[n_var1].val=z;
				}
				n_var1++;
				n_lev1+=model.term[k].df;
			}
			if(n_var1!=t_var || n_lev1!=t_lev) {
				fprintf(stderr,"%d %d  %d %d\n",t_var,n_var1,t_lev,n_lev1);
				ABT_FUNC("Internal error - level mismatch\n");
			}
			id_array[i].res[rec]=y;
			entry[0].pos=0;
			entry[0].val=y;
			/* Add contributions to X'X */
			/* Reorder equations */
			for(k=b_var;k<t_var;k++) {
				k1=entry[k].pos;
				if(k1>=0) {
					if(k1>=b_lev) { 
						k2=k1;
						k1=n_lev+b_lev-order1[(k1-b_lev)];
					} else ABT_FUNC("Illegal equation position\n");
					if(k1<0 || k1>=t_lev) ABT_FUNC("Illegal equation position\n");
				}
				entry[k].pos=k1;
			}
			for(k=0;k<t_var;k++) {
				k1=entry[k].pos;
				if(k1>=0) {
					z=entry[k].val;
					if(z<DBL_EPSILON) continue;
					XX[k1].val+=z*z/(wt*residual_var);
					for(k2=k+1;k2<t_var;k2++) if((k3=entry[k2].pos)>=0) {
						y=entry[k2].val;
						add_XX_entry(k3,k1,z*y/(wt*residual_var));
					}
				}
			}
			lk-=.5*log(2.0*M_PI*wt*residual_var);
		}
	}
	if(polygenic_flag) {
		z=1.0/additive_var;
		/* Add A^-1 Matrix */
		y=0.0;
		j=poly_start;
		for(comp=0;comp<n_comp;comp++) {
			AI=AIMatrix[comp];
			for(i=0;i<comp_size[comp];i++) {
				k=n_lev+b_lev-order1[j+i];
				XX[k].val+=z*AI[i].val;
				for(k1=AI[i].x;k1<AI[i+1].x;k1++) {
					k2=AI[k1].x+j;
					k2=n_lev+b_lev-order1[k2];
					add_XX_entry(k,k2,z*AI[k1].val);
				}
			}
			j+=i;
			/* Accumulate determinant of A^-1 */
			y+=AI[i].val;
		}
		lk-=.5*(log(2.0*M_PI/z)*(double)ped_size+y);
	}
	/* Add random terms for additional random effects */
	if(n_random) for(j=rand_start,i=k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_RANDOM) {
			z=1.0/c_var[i];
			for(k1=0;k1<model.term[k].df;k1++) {
				k2=n_lev+b_lev-order1[j++];
				XX[k2].val+=z;
			}
			lk-=.5*log(2.0*M_PI/z)*(double)k1;
			i++;
		}
	}
	nn=0;
	z=1.0/tau;
	/* Add random terms for QTLs */
	if(n_qt) {
		for(j=qt_start,i=0;i<n_tloci;i++) if(tlocus[i].flag)	{
			k2=tlocus[i].n_alleles;
			k2=(no_overdominant&&k2==2)?1:k2*(k2+1)/2-1;
			for(k=0;k<k2;k++) XX[j++].val+=z;
			nn+=k2;
		}
	}
	/* and for candidate genes */
	if(mk_start>=0) {
		for(j=mk_start,k=0;k<model.n_terms;k++) {
			type=model.term[k].vars[0].type;
			if(type&ST_MARKER) {
				for(k1=0;k1<model.term[k].df;k1++) {
					k2=n_lev+b_lev-order1[j++];
					XX[k2].val+=z;
				}
				nn+=k1;
			}
		}
	}
	if(nn) lk-=.5*(double)nn*log(2.0*M_PI/z);
	det=0.0;
	nn=0;
	/* Gaussian elimination step */
	for(i=t_lev-1;i>0;i--) {
		y=XX[i].val;
		if(fabs(y)<1.0e-12) {
			zero[i]=1;
			continue;
		} else zero[i]=0;
		if(y<DBL_EPSILON) {
			ABT_FUNC("Effects matrix not positive definite\n");
		}
		det+=log(y);
		nn++;
		y=1.0/y;
		/* Absorb pivot row into remaining rows */
		if((p=XX[i].First)) {
			XX[p->col].val-=p->val*p->val*y;
			while(p->Next)
			{
				p=p->Next;
				p1=XX[i].First;
				z=-p->val*y;
				while(p!=p1) {
					add_XX_entry(p->col,p1->col,p1->val*z);
					p1=p1->Next;
				}
				XX[p->col].val-=p->val*p->val*y;
			}
		}
	}
	lk+=.5*((double)nn*log(2.0*M_PI)-det-XX[0].val);
	/* And now go backwards, sampling as we go */
	for(i=1;i<t_lev;i++) if(!zero[i]) {
		if((p=XX[i].First)) {
			p1=p;
			if(!p->col) {
				y=p->val;
				p=p->Next;
			} else y=0.0;
			while(p) {
				j=p->col;
				if(!zero[j]) y-=B[j]*p->val;
				p1=p;
				p=p->Next;
			}
			/* Free up what we don't need */
			if(p1) {
				p1->Next=freelist;
				freelist=XX[i].First;
			}
		} else y=0.0;
		z=XX[i].val;
  		B[i]=y/z+snorm()*sqrt(1.0/z);
		if(no_overdominant && n_qt && i>qt_start && i<qt_start+n_qtlev) {
			for(j=qt_start,k=0;i>j && k<n_tloci;k++) if(tlocus[k].flag)	{
				k2=tlocus[k].n_alleles;
				k1=k2*(k2+1)/2-1;
				if(k2==2) {
					if(i==j+k2-1) {
						z=B[i]?B[i-1]/B[i]:0.0;
						if(z<0.0 || z>1.0) {
							B[i-1]=tlocus[k].eff[0];
							B[i]=tlocus[k].eff[1];
						}
						break;
					}
				}
				j+=k1;
			}
		}
	}
	/* Store new effect samples */
	/* Grand Mean */
	if(grand_mean_set!=1) {
		if(!zero[1]) grand_mean=B[1];
	}
	/* QTLs */
	if(n_qt) {
		for(j=qt_start,i=0;i<n_tloci;i++) if(tlocus[i].flag)	{
			k2=tlocus[i].n_alleles;
			k2=k2*(k2+1)/2-1;
			for(k=0;k<k2;k++) tlocus[i].eff[k]=B[j++];
			nn+=k2;
		}
	}
	/* candidate genes */
	if(mk_start>=0) {
		for(j=mk_start,k=0;k<model.n_terms;k++) {
			type=model.term[k].vars[0].type;
			if(type&ST_MARKER) {
				for(k1=0;k1<model.term[k].df;k1++) {
					k2=n_lev+b_lev-order1[j++];
					model.term[k].eff[k1]=B[k2];
				}
			}
		}
	}
	/* additional random effects */
	if(n_random) for(j=rand_start,i=k=0;k<model.n_terms;k++) {
		type=model.term[k].vars[0].type;
		if(type&ST_RANDOM) {
			for(k1=0;k1<model.term[k].df;k1++) {
				k2=n_lev+b_lev-order1[j++];
				model.term[k].eff[k1]=B[k2];
			}
		}
	}
	/* Polygenic component */
	if(polygenic_flag) {
		j=poly_start;
		for(i=0;i<ped_size;i++) {
			k=n_lev+b_lev-order1[j++];
			id_array[i].bv=B[k];
		}
	}
	/* and other covariates effects */
	if(cov_start>=0) {
		j=cov_start;
		for(k=0;k<model.n_terms;k++) {
			type=model.term[k].vars[0].type;
			if(type&(ST_TRAITLOCUS|ST_ID|ST_MARKER|ST_RANDOM)) continue; /* Already done */
			for(k1=0;k1<model.term[k].df;k1++) {
				k2=n_lev+b_lev-order1[j++];
				model.term[k].eff[k1]=B[k2];
			}
		}
	}
	ss=ssn=0.0;
	/* Correct residuals for new effects */
	for(i=0;i<ped_size;i++) if(id_array[i].res) {
		nrec=id_array[i].n_rec;
		if(!nrec) nrec=1;
		for(rec=0;rec<nrec;rec++) {
			if(mtype&ST_CONSTANT) y=id_array[i].res[0];
			else y=id_array[i].res[rec];
			if(grand_mean_set!=1) y-=grand_mean;
			if(polygenic_flag) y-=id_array[i].bv;
			for(j=0;j<n_tloci;j++) if(tlocus[j].flag)	{
				k2=tlocus[j].gt[i]-1;
				if(k2) y-=tlocus[j].eff[k2-1];
			}
			for(k=0;k<model.n_terms;k++) {
				type=model.term[k].vars[0].type;
				if(type&(ST_ID|ST_TRAITLOCUS)) continue;
				k1=model.term[k].vars[0].var_index;
				if(type&ST_MARKER) {
					k2=marker[k1].gt[i]-1;
					if(k2) y-=model.term[k].eff[k2-1];
				} else {
					data=0;
					if(type&ST_CONSTANT)	{
						if(id_array[i].data) data=id_array[i].data+k1;
					} else if(id_array[i].data1) data=id_array[i].data1[rec]+k1;
					if(!data) ABT_FUNC("Internal error - null data pointer\n");
					if(type&ST_FACTOR) {
						k2=(int)data->data.value-1;
						if(!(type&ST_RANDOM)) k2--;
						if(k2>=0) y-=model.term[k].eff[k2];
					} else {
						if(data->flag&ST_INTTYPE) z=(double)data->data.value;
						else z=data->data.rvalue;
						y-=model.term[k].eff[0]*z;
					}
				}
			}
			id_array[i].res[rec]=y;
			ss+=y*y;
			ssn+=1.0;
		}
	}
	return lk;
}
